@@ -23,10 +23,13 @@
 
 #define G_LOG_DOMAIN "chatty-application"
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <glib/gi18n.h>
 #include <handy.h>
 
-#include "chatty-config.h"
 #include "chatty-window.h"
 #include "chatty-utils.h"
 #include "users/chatty-pp-account.h"
@@ -34,6 +37,7 @@
 #include "chatty-application.h"
 #include "chatty-settings.h"
 #include "chatty-history.h"
+#include "chatty-log.h"
 
 #define LIBFEEDBACK_USE_UNSTABLE_API
 #include <libfeedback.h>
@@ -51,29 +55,50 @@ struct _ChattyApplication
 
   GtkWidget      *main_window;
   ChattySettings *settings;
-  GtkCssProvider *css_provider;
   ChattyManager  *manager;
 
   char *uri;
   guint open_uri_id;
 
+  gulong   delete_id;
+  gulong   open_chat_id;
+
   gboolean daemon;
   gboolean show_window;
   gboolean enable_debug;
-  gboolean enable_verbose;
 };
 
 G_DEFINE_TYPE (ChattyApplication, chatty_application, GTK_TYPE_APPLICATION)
 
+static gboolean    cmd_verbose_cb   (const char *option_name,
+                                     const char *value,
+                                     gpointer    data,
+                                     GError     **error);
+
 static GOptionEntry cmd_options[] = {
-  { "version", 'v', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, NULL, N_("Show release version"), NULL },
+  { "version", 0, G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, NULL, N_("Show release version"), NULL },
   { "daemon", 'D', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, NULL, N_("Start in daemon mode"), NULL },
   { "nologin", 'n', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, NULL, N_("Disable all accounts"), NULL },
   { "debug", 'd', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, NULL, N_("Enable libpurple debug messages"), NULL },
-  { "verbose", 'V', G_OPTION_FLAG_NONE, G_OPTION_ARG_NONE, NULL, N_("Enable verbose libpurple debug messages"), NULL },
+  { "verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, cmd_verbose_cb,
+    N_("Enable verbose libpurple debug messages"), NULL },
   { NULL }
 };
 
+static gboolean
+cmd_verbose_cb (const char  *option_name,
+                const char  *value,
+                gpointer     data,
+                GError     **error)
+{
+  chatty_log_increase_verbosity ();
+
+  purple_debug_set_enabled (TRUE);
+  purple_debug_set_verbose (TRUE);
+
+  return TRUE;
+}
+
 static int
 run_dialog_and_destroy (GtkDialog *dialog)
 {
@@ -168,7 +193,7 @@ application_show_connection_error (ChattyApplication *self,
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG(dialog),
                                             "%s: %s\n\n%s",
                                             message,
-                                            chatty_account_get_username (CHATTY_ACCOUNT (account)),
+                                            chatty_item_get_username (CHATTY_ITEM (account)),
                                             _("Please check ID and password"));
 
   run_dialog_and_destroy (GTK_DIALOG (dialog));
@@ -181,11 +206,7 @@ application_open_chat (ChattyApplication *self,
   g_assert (CHATTY_IS_APPLICATION (self));
   g_assert (CHATTY_IS_CHAT (chat));
 
-  if (!self->main_window) {
-    self->main_window = chatty_window_new (GTK_APPLICATION (self));
-    g_object_add_weak_pointer (G_OBJECT (self->main_window), (gpointer *)&self->main_window);
-  }
-
+  g_application_activate (G_APPLICATION (self));
   chatty_window_open_chat (CHATTY_WINDOW (self->main_window), chat);
 }
 
@@ -211,8 +232,38 @@ chatty_application_show_window (GSimpleAction *action,
 
   g_assert (CHATTY_IS_APPLICATION (self));
 
+  self->show_window = TRUE;
   g_application_activate (G_APPLICATION (self));
-  gtk_window_present (GTK_WINDOW (self->main_window));
+}
+
+static void
+chatty_application_open_chat (GSimpleAction *action,
+                              GVariant      *parameter,
+                              gpointer       user_data)
+{
+  ChattyApplication *self = user_data;
+  g_autoptr(GString) str = NULL;
+  const char *room_id, *account_id;
+  ChattyChat *chat;
+
+  g_assert (CHATTY_IS_APPLICATION (self));
+
+  g_variant_get (parameter, "(ss)", &room_id, &account_id);
+  g_return_if_fail (room_id && account_id);
+
+  chat = chatty_manager_find_chat_with_name (self->manager, account_id, room_id);
+  g_return_if_fail (chat);
+
+  str = g_string_new (NULL);
+  g_string_append (str, "Opening chat:");
+  chatty_log_anonymize_value (str, room_id);
+  g_string_append (str, ", account:");
+  chatty_log_anonymize_value (str, account_id);
+  CHATTY_DEBUG_MSG ("%s", str->str);
+
+  self->show_window = TRUE;
+  g_application_activate (G_APPLICATION (self));
+  chatty_window_open_chat (CHATTY_WINDOW (self->main_window), chat);
 }
 
 static void
@@ -220,8 +271,10 @@ chatty_application_finalize (GObject *object)
 {
   ChattyApplication *self = (ChattyApplication *)object;
 
+  g_clear_signal_handler (&self->open_chat_id, self->manager);
+  g_clear_signal_handler (&self->delete_id, self->main_window);
+
   g_clear_handle_id (&self->open_uri_id, g_source_remove);
-  g_clear_object (&self->css_provider);
   g_clear_object (&self->manager);
 
   G_OBJECT_CLASS (chatty_application_parent_class)->finalize (object);
@@ -262,17 +315,14 @@ chatty_application_command_line (GApplication            *application,
     g_debug ("Enable daemon mode");
   }
 
-  if (g_variant_dict_contains (options, "nologin")) {
+  if (g_variant_dict_contains (options, "nologin"))
     chatty_manager_disable_auto_login (chatty_manager_get_default (), TRUE);
-  } else if (g_variant_dict_contains (options, "debug")) {
+
+  if (g_variant_dict_contains (options, "debug"))
     self->enable_debug = TRUE;
-  } else if (g_variant_dict_contains (options, "verbose")) {
-    self->enable_debug = TRUE;
-    self->enable_verbose = TRUE;
-  }
 
   purple_debug_set_enabled (self->enable_debug);
-  purple_debug_set_verbose (self->enable_verbose);
+  purple_debug_set_verbose (chatty_log_get_verbosity () > 0);
 
   arguments = g_application_command_line_get_arguments (command_line, &argc);
 
@@ -292,36 +342,47 @@ static void
 chatty_application_startup (GApplication *application)
 {
   ChattyApplication *self = (ChattyApplication *)application;
+  g_autoptr(GtkCssProvider) provider = NULL;
   g_autofree char *db_path = NULL;
+  g_autofree char *dir = NULL;
   static const GActionEntry app_entries[] = {
+    { "open-chat", chatty_application_open_chat, "(ss)" },
     { "show-window", chatty_application_show_window },
   };
 
   self->daemon = FALSE;
-  self->manager = g_object_ref (chatty_manager_get_default ());
+  self->manager = chatty_manager_get_default ();
 
   G_APPLICATION_CLASS (chatty_application_parent_class)->startup (application);
 
+  g_info ("%s %s, git version: %s", PACKAGE_NAME, PACKAGE_VERSION, GIT_VERSION);
+
   hdy_init ();
 
   g_set_application_name (_("Chats"));
 
+  dir = g_build_filename (g_get_user_cache_dir (), "chatty", "matrix", "files", "thumbnail", NULL);
+  g_mkdir_with_parents (dir, S_IRWXU);
+
   lfb_init (CHATTY_APP_ID, NULL);
   db_path =  g_build_filename (purple_user_dir(), "chatty", "db", NULL);
-  chatty_history_open (db_path, "chatty-history.db");
+  chatty_history_open (chatty_manager_get_history (self->manager),
+                       db_path, "chatty-history.db");
 
   self->settings = chatty_settings_get_default ();
+  if (chatty_settings_get_experimental_features (self->settings))
+    g_warning ("Experimental features enabled");
 
-  self->css_provider = gtk_css_provider_new ();
-  gtk_css_provider_load_from_resource (self->css_provider,
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_resource (provider,
                                        "/sm/puri/Chatty/css/style.css");
+  gtk_style_context_add_provider_for_screen (gdk_screen_get_default(),
+                                             GTK_STYLE_PROVIDER (provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
 
   g_action_map_add_action_entries (G_ACTION_MAP (self), app_entries,
                                    G_N_ELEMENTS (app_entries), self);
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default(),
-                                             GTK_STYLE_PROVIDER (self->css_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
   g_signal_connect_object (self->manager, "authorize-buddy",
                            G_CALLBACK (application_authorize_buddy_cb), self,
                            G_CONNECT_SWAPPED);
@@ -349,15 +410,15 @@ chatty_application_activate (GApplication *application)
     g_object_add_weak_pointer (G_OBJECT (self->main_window), (gpointer *)&self->main_window);
   }
 
-  if (self->daemon)
-    g_signal_connect (G_OBJECT (self->main_window),
-                      "delete-event",
-                      G_CALLBACK (gtk_widget_hide_on_delete),
-                      NULL);
+  if (self->daemon && !self->delete_id)
+    self->delete_id = g_signal_connect (self->main_window, "delete-event",
+                                        G_CALLBACK (gtk_widget_hide_on_delete),
+                                        NULL);
 
-  g_signal_connect_object (self->manager, "open-chat",
-                           G_CALLBACK (application_open_chat), self,
-                           G_CONNECT_SWAPPED);
+  if (!self->open_chat_id)
+    self->open_chat_id = g_signal_connect_swapped (self->manager, "open-chat",
+                                                   G_CALLBACK (application_open_chat),
+                                                   self);
 
   if (self->show_window)
     gtk_window_present (GTK_WINDOW (self->main_window));
@@ -372,8 +433,10 @@ chatty_application_activate (GApplication *application)
 static void
 chatty_application_shutdown (GApplication *application)
 {
+  ChattyApplication *self = (ChattyApplication *)application;
+
   g_object_unref (chatty_settings_get_default ());
-  chatty_history_close ();
+  chatty_history_close (chatty_manager_get_history (self->manager));
   lfb_uninit ();
 
   G_APPLICATION_CLASS (chatty_application_parent_class)->shutdown (application);
@@ -412,17 +475,6 @@ chatty_application_new (void)
                        NULL);
 }
 
-ChattyWindow *
-chatty_application_get_main_window (ChattyApplication *self)
-{
-  g_return_val_if_fail (CHATTY_IS_APPLICATION (self), NULL);
-
-  if (self->main_window)
-    return CHATTY_WINDOW (self->main_window);
-
-  return NULL;
-}
-
 /**
  * chatty_application_get_active_chat:
  * @self: A #ChattyApplication
