@@ -17,9 +17,8 @@ G_BEGIN_DECLS
 
 #define CHATTY_TYPE_AVATAR (chatty_avatar_get_type ())
 
-G_DECLARE_FINAL_TYPE (ChattyAvatar, chatty_avatar, CHATTY, AVATAR, GtkImage)
+G_DECLARE_FINAL_TYPE (ChattyAvatar, chatty_avatar, CHATTY, AVATAR, GtkBin)
 
-GtkWidget *chatty_avatar_new      (ChattyItem   *item);
 void       chatty_avatar_set_title (ChattyAvatar *item,
                                     const char   *title);
 void       chatty_avatar_set_item (ChattyAvatar *self,
