@@ -10,15 +10,17 @@
  */
 
 #include <glib/gi18n.h>
+#include <gspell/gspell.h>
 
 #include "chatty-avatar.h"
 #include "chatty-chat.h"
+#include "chatty-mm-chat.h"
 #include "chatty-pp-chat.h"
 #include "chatty-history.h"
-#include "chatty-icons.h"
-#include "chatty-manager.h"
 #include "chatty-utils.h"
+#include "chatty-enums.h"
 #include "chatty-window.h"
+#include "matrix/chatty-ma-chat.h"
 #include "users/chatty-contact.h"
 #include "users/chatty-pp-buddy.h"
 #include "chatty-message-row.h"
@@ -29,6 +31,7 @@ struct _ChattyChatView
   GtkBox      parent_instance;
 
   GtkWidget  *message_list;
+  GtkWidget  *loading_spinner;
   GtkWidget  *typing_revealer;
   GtkWidget  *typing_indicator;
   GtkWidget  *chatty_message_list;
@@ -41,49 +44,28 @@ struct _ChattyChatView
   GtkWidget  *empty_view;
   GtkWidget  *empty_label0;
   GtkWidget  *empty_label1;
-  GtkWidget  *empty_label2;
+  GtkWidget  *scroll_down_button;
   GtkTextBuffer *message_input_buffer;
   GtkAdjustment *vadjustment;
 
+  /* Signal ids */
+  GBinding   *history_binding;
+
   ChattyChat *chat;
-  PurpleConversation *conv;
   char       *last_message_id;  /* id of last sent message, currently used only for SMS */
-  guint       message_type;
   guint       refresh_typing_id;
   gboolean    first_scroll_to_bottom;
 };
 
-static GHashTable *ht_sms_id = NULL;
-
 #define INDICATOR_WIDTH   60
 #define INDICATOR_HEIGHT  40
 #define INDICATOR_MARGIN   2
 #define MSG_BUBBLE_MAX_RATIO .3
 
-#define LAZY_LOAD_INITIAL_MSGS_LIMIT 20
-
 G_DEFINE_TYPE (ChattyChatView, chatty_chat_view, GTK_TYPE_BOX)
 
 
-const char *disclaimer_strings[][3] = {
-  {
-    N_("This is an IM conversation."),
-    N_("Your messages are not encrypted,"),
-    N_("ask your counterpart to use E2EE."),
-  },
-  {
-    N_("This is an IM conversation."),
-    N_("Your messages are secured"),
-    N_("by end-to-end encryption."),
-  },
-  {
-    N_("This is an SMS conversation."),
-    N_("Your messages are not encrypted,"),
-    N_("and carrier rates may apply."),
-  },
-};
-
-const char *emoticons[][15] = {
+const char *emoticons[][2] = {
   {":)", "🙂"},
   {";)", "😉"},
   {":(", "🙁"},
@@ -101,35 +83,12 @@ const char *emoticons[][15] = {
   {"FROSTY", "⛄"},
 };
 
-static gboolean
-chat_view_time_is_same_day (time_t time_a,
-                            time_t time_b)
-{
-  struct tm *tm;
-  int day_a, day_b;
+enum {
+  FILE_REQUESTED,
+  N_SIGNALS
+};
 
-  if (difftime (time_a, time_b) > SECONDS_PER_DAY)
-    return FALSE;
-
-  tm = localtime (&time_a);
-  day_a = tm->tm_yday;
-
-  tm = localtime (&time_b);
-  day_b = tm->tm_yday;
-
-  if (day_a == day_b)
-    return TRUE;
-
-  return FALSE;
-}
-
-static gboolean
-chat_view_hash_table_match_item (gpointer key,
-                                 gpointer value,
-                                 gpointer user_data)
-{
-  return value == user_data;
-}
+static guint signals[N_SIGNALS];
 
 static void
 chatty_draw_typing_indicator (cairo_t *cr)
@@ -222,131 +181,71 @@ chatty_check_for_emoticon (ChattyChatView *self)
     }
 }
 
-static void
-chat_view_setup_file_upload (ChattyChatView *self)
-{
-  PurplePluginProtocolInfo *prpl_info;
-  PurpleConnection         *gc;
-  PurpleBlistNode          *node;
-  g_autoptr(GList)          list = NULL;
-
-  g_assert (CHATTY_IS_CHAT_VIEW (self));
-
-  gtk_widget_show (self->send_file_button);
-
-  gc = purple_conversation_get_gc (self->conv);
-  node = chatty_utils_get_conv_blist_node (self->conv);
-  prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO (gc->prpl);
-
-  if (prpl_info->blist_node_menu)
-    list = prpl_info->blist_node_menu (node);
-
-  for (GList *l = list; l; l = l->next) {
-    PurpleMenuAction *act = l->data;
-
-    if (g_strcmp0 (act->label, "HTTP File Upload") == 0) {
-      g_object_set_data (G_OBJECT (self->send_file_button),
-                         "callback", act->callback);
-
-      g_object_set_data (G_OBJECT (self->send_file_button),
-                         "callback-data", act->data);
-    }
-    purple_menu_action_free (act);
-  }
-}
-
 static void
 chatty_chat_view_update (ChattyChatView *self)
 {
   GtkStyleContext *context;
-  int index = -1;
-
+  ChattyProtocol protocol;
 
   g_assert (CHATTY_IS_CHAT_VIEW (self));
 
-  if (chatty_chat_is_im (self->chat) &&
-      chatty_item_get_protocols (CHATTY_ITEM (self->chat)) == CHATTY_PROTOCOL_SMS)
-    self->message_type = CHATTY_MSG_TYPE_SMS;
-  else if (chatty_chat_is_im (self->chat))
-    self->message_type = CHATTY_MSG_TYPE_IM;
-  else
-    self->message_type = CHATTY_MSG_TYPE_MUC;
+  protocol = chatty_item_get_protocols (CHATTY_ITEM (self->chat));
 
   gtk_widget_show (self->encrypt_icon);
 
   if (chatty_chat_is_im (self->chat) && CHATTY_IS_PP_CHAT (self->chat))
     chatty_pp_chat_load_encryption_status (CHATTY_PP_CHAT (self->chat));
 
-  if (chatty_manager_has_file_upload_plugin (chatty_manager_get_default ()) &&
-      chatty_item_get_protocols (CHATTY_ITEM (self->chat)) == CHATTY_PROTOCOL_XMPP)
-    chat_view_setup_file_upload (self);
+  gtk_widget_set_visible (self->send_file_button, chatty_chat_has_file_upload (self->chat));
 
-  if (self->message_type == CHATTY_MSG_TYPE_IM)
-    index = 0;
-  else if (self->message_type == CHATTY_MSG_TYPE_SMS)
-    index = 2;
-
-  /* XXX: This is temporary, strings put into different labels isn’t good for translation */
-  if (index >= 0) {
-    gtk_label_set_label (GTK_LABEL (self->empty_label0), disclaimer_strings[index][0]);
-    gtk_label_set_label (GTK_LABEL (self->empty_label1), disclaimer_strings[index][1]);
-    gtk_label_set_label (GTK_LABEL (self->empty_label2), disclaimer_strings[index][2]);
+  if (protocol == CHATTY_PROTOCOL_MMS_SMS) {
+    gtk_label_set_label (GTK_LABEL (self->empty_label0),
+                         _("This is an SMS conversation"));
+    gtk_label_set_label (GTK_LABEL (self->empty_label1),
+                         _("Your messages are not encrypted, "
+                           "and carrier rates may apply"));
+  } else if (chatty_chat_is_im (self->chat)) {
+    gtk_label_set_label (GTK_LABEL (self->empty_label0),
+                         _("This is an IM conversation"));
+    if (chatty_chat_get_encryption (self->chat) == CHATTY_ENCRYPTION_ENABLED)
+      gtk_label_set_label (GTK_LABEL (self->empty_label1),
+                           _("Your messages are encrypted"));
+    else
+      gtk_label_set_label (GTK_LABEL (self->empty_label1),
+                           _("Your messages are not encrypted"));
   }
 
   context = gtk_widget_get_style_context (self->send_message_button);
 
-  if (self->message_type == CHATTY_MSG_TYPE_SMS)
+  if (protocol == CHATTY_PROTOCOL_MMS_SMS)
     gtk_style_context_add_class (context, "button_send_green");
-  else if (self->message_type == CHATTY_MSG_TYPE_IM)
+  else if (chatty_chat_is_im (self->chat))
     gtk_style_context_add_class (context, "suggested-action");
 }
 
 static void
 chatty_update_typing_status (ChattyChatView *self)
 {
-  PurpleConversation     *conv;
-  PurpleConvIm           *im;
   GtkTextIter             start, end;
   g_autofree char         *text = NULL;
   gboolean                empty;
 
   g_assert (CHATTY_IS_CHAT_VIEW (self));
 
-  conv = self->conv;
-
-  if (!chatty_chat_is_im (self->chat))
-    return;
-
   gtk_text_buffer_get_bounds (self->message_input_buffer, &start, &end);
   text = gtk_text_buffer_get_text (self->message_input_buffer, &start, &end, FALSE);
 
   empty = !text || !*text || *text == '/';
+  chatty_chat_set_typing (self->chat, !empty);
+}
 
-  im = PURPLE_CONV_IM (conv);
+static void
+chat_view_scroll_down_clicked_cb (ChattyChatView *self)
+{
+  g_assert (CHATTY_IS_CHAT_VIEW (self));
 
-  if (!empty) {
-    gboolean send = (purple_conv_im_get_send_typed_timeout (im) == 0);
-
-    purple_conv_im_stop_send_typed_timeout (im);
-    purple_conv_im_start_send_typed_timeout (im);
-
-    if (send || (purple_conv_im_get_type_again (im) != 0 &&
-                 time (NULL) > purple_conv_im_get_type_again (im))) {
-      unsigned int timeout;
-
-      timeout = serv_send_typing (purple_conversation_get_gc (conv),
-                                  purple_conversation_get_name (conv),
-                                  PURPLE_TYPING);
-
-      purple_conv_im_set_type_again (im, timeout);
-    }
-  } else {
-    purple_conv_im_stop_send_typed_timeout (im);
-
-    serv_send_typing (purple_conversation_get_gc (conv),
-                      purple_conversation_get_name (conv),
-                      PURPLE_NOT_TYPING);
-  }
+  gtk_adjustment_set_value (self->vadjustment,
+                            gtk_adjustment_get_upper (self->vadjustment));
 }
 
 static void
@@ -356,10 +255,25 @@ chat_view_edge_overshot_cb (ChattyChatView  *self,
   g_assert (CHATTY_IS_CHAT_VIEW (self));
 
   if (pos == GTK_POS_TOP)
-    chatty_manager_load_more_chat (chatty_manager_get_default (), self->chat, LAZY_LOAD_INITIAL_MSGS_LIMIT);
+    chatty_chat_load_past_messages (self->chat, -1);
 }
 
 
+static void
+chat_account_status_changed_cb (ChattyChatView *self)
+{
+  ChattyAccount *account;
+  gboolean enabled;
+
+  account = chatty_chat_get_account (self->chat);
+  g_return_if_fail (account);
+
+  enabled = chatty_account_get_status (account) == CHATTY_CONNECTED;
+  gtk_widget_set_sensitive (self->message_input, enabled);
+  gtk_widget_set_sensitive (self->send_file_button, enabled);
+  gtk_widget_set_sensitive (self->send_message_button, enabled);
+}
+
 static GtkWidget *
 chat_view_message_row_new (ChattyMessage  *message,
                            ChattyChatView *self)
@@ -378,50 +292,6 @@ chat_view_message_row_new (ChattyMessage  *message,
   return GTK_WIDGET (row);
 }
 
-static void
-messages_items_changed_cb (ChattyChatView *self,
-                           guint           position,
-                           guint           removed,
-                           guint           added)
-{
-  ChattyMessage *next_msg, *msg;
-  GtkListBoxRow *row, *next_row;
-  GtkListBox *list;
-  time_t next_time, time;
-
-  g_assert (CHATTY_IS_CHAT_VIEW (self));
-
-  if (added == 0)
-    return;
-
-  /* Don't hide footers in group chats */
-  if (self->message_type == CHATTY_MSG_TYPE_MUC)
-    return;
-
-  list = GTK_LIST_BOX (self->message_list);
-
-  for (gint i = position; i < position + added; i++) {
-    next_row = gtk_list_box_get_row_at_index (list, i + 1);
-
-    if (!next_row)
-      break;
-
-    row = gtk_list_box_get_row_at_index (list, i);
-    msg = chatty_message_row_get_item (CHATTY_MESSAGE_ROW (row));
-    time = chatty_message_get_time (msg);
-
-    next_msg = chatty_message_row_get_item (CHATTY_MESSAGE_ROW (next_row));
-    next_time = chatty_message_get_time (next_msg);
-
-    /*
-     * Hide time if the message following the current one belong to the same
-     * day
-     */
-    if (chat_view_time_is_same_day (time, next_time))
-      chatty_message_row_hide_footer (CHATTY_MESSAGE_ROW (row));
-  }
-}
-
 static void
 chat_encrypt_changed_cb (ChattyChatView *self)
 {
@@ -492,171 +362,47 @@ chat_view_input_focus_out_cb (ChattyChatView *self)
   return FALSE;
 }
 
-
-
-static gboolean
-chatty_conv_check_for_command (ChattyChatView *self)
-{
-  PurpleConversation *conv;
-  gchar              *cmd;
-  gboolean            retval = FALSE;
-  GtkTextIter         start, end;
-  PurpleMessageFlags  flags = 0;
-
-  flags |= PURPLE_MESSAGE_NO_LOG | PURPLE_MESSAGE_SYSTEM;
-  conv = self->conv;
-
-  gtk_text_buffer_get_bounds (self->message_input_buffer, &start, &end);
-
-  cmd = gtk_text_buffer_get_text (self->message_input_buffer, &start, &end, FALSE);
-
-  if (cmd && *cmd == '/') {
-    PurpleCmdStatus status;
-    gchar *error, *cmdline;
-
-    cmdline = cmd + strlen ("/");
-
-    if (purple_strequal (cmdline, "xyzzy")) {
-      purple_conversation_write (conv,
-                                 "",
-                                 "Nothing happens",
-                                 flags,
-                                 time(NULL));
-
-      g_free (cmd);
-      return TRUE;
-    }
-
-    purple_conversation_write (conv,
-                               "",
-                               cmdline,
-                               flags,
-                               time(NULL));
-
-    status = purple_cmd_do_command (conv, cmdline, cmdline, &error);
-
-    switch (status) {
-    case PURPLE_CMD_STATUS_OK:
-      retval = TRUE;
-      break;
-    case PURPLE_CMD_STATUS_NOT_FOUND:
-      {
-        PurplePluginProtocolInfo *prpl_info = NULL;
-        PurpleConnection *gc;
-
-        if ((gc = purple_conversation_get_gc (conv)))
-          prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl);
-
-        if ((prpl_info != NULL) &&
-            (prpl_info->options & OPT_PROTO_SLASH_COMMANDS_NATIVE)) {
-          gchar *spaceslash;
-
-          /* If the first word in the entered text has a '/' in it, then the user
-           * probably didn't mean it as a command. So send the text as message. */
-          spaceslash = cmdline;
-
-          while (*spaceslash && *spaceslash != ' ' && *spaceslash != '/') {
-            spaceslash++;
-          }
-
-          if (*spaceslash != '/') {
-            purple_conversation_write (conv,
-                                       "",
-                                       "Unknown command. Get a list of available commands with '/chatty help'",
-                                       flags,
-                                       time(NULL));
-            retval = TRUE;
-          }
-        }
-        break;
-      }
-    case PURPLE_CMD_STATUS_WRONG_ARGS:
-      purple_conversation_write (conv,
-                                 "",
-                                 "Wrong number of arguments for the command.",
-                                 flags,
-                                 time(NULL));
-      retval = TRUE;
-      break;
-    case PURPLE_CMD_STATUS_FAILED:
-      purple_conversation_write (conv,
-                                 "",
-                                 error ? error : "The command failed.",
-                                 flags,
-                                 time(NULL));
-      g_free(error);
-      retval = TRUE;
-      break;
-    case PURPLE_CMD_STATUS_WRONG_TYPE:
-      if (purple_conversation_get_type (conv) == PURPLE_CONV_TYPE_IM)
-        purple_conversation_write (conv,
-                                   "",
-                                   "That command only works in chats, not IMs.",
-                                   flags,
-                                   time(NULL));
-      else
-        purple_conversation_write (conv,
-                                   "",
-                                   "That command only works in IMs, not chats.",
-                                   flags,
-                                   time(NULL));
-      retval = TRUE;
-      break;
-    case PURPLE_CMD_STATUS_WRONG_PRPL:
-      purple_conversation_write (conv,
-                                 "",
-                                 "That command doesn't work on this protocol.",
-                                 flags,
-                                 time(NULL));
-      retval = TRUE;
-      break;
-    default:
-      break;
-    }
-  }
-
-  g_free (cmd);
-  return retval;
-}
-
-
 static void
 chat_view_send_file_button_clicked_cb (ChattyChatView *self,
                                        GtkButton      *button)
 {
-  PurpleBlistNode *node;
-  gpointer data;
-
-  void (*callback)(gpointer, gpointer);
-
   g_assert (CHATTY_IS_CHAT_VIEW (self));
   g_assert (GTK_IS_BUTTON (button));
+  g_return_if_fail (chatty_chat_has_file_upload (self->chat));
 
-  callback = g_object_get_data (G_OBJECT (button), "callback");
-  data = g_object_get_data (G_OBJECT (button), "callback-data");
-  node = chatty_utils_get_conv_blist_node (self->conv);
+  if (CHATTY_IS_MA_CHAT (self->chat)) {
+    /* TODO */
 
-  if (callback)
-    callback (node, data);
+  } else if (CHATTY_IS_PP_CHAT (self->chat)) {
+    chatty_pp_chat_show_file_upload (CHATTY_PP_CHAT (self->chat));
+  }
+}
+
+static void
+view_send_message_async_cb (GObject      *object,
+                            GAsyncResult *result,
+                            gpointer      user_data)
+{
+  g_autoptr(ChattyChatView) self = user_data;
+
+  chatty_chat_set_unread_count (self->chat, 0);
 }
 
 static void
 chat_view_send_message_button_clicked_cb (ChattyChatView *self)
 {
-  PurpleConversation  *conv;
   ChattyAccount *account;
+  g_autoptr(ChattyMessage) msg = NULL;
+  g_autofree char *message = NULL;
   GtkTextIter    start, end;
-  gchar         *message = NULL;
-  gchar         *sms_id_str;
-  guint          sms_id;
 
   g_assert (CHATTY_IS_CHAT_VIEW (self));
 
-  conv = self->conv;
-
   gtk_text_buffer_get_bounds (self->message_input_buffer, &start, &end);
+  message = gtk_text_buffer_get_text (self->message_input_buffer, &start, &end, FALSE);
 
-  if (chatty_conv_check_for_command (self)) {
+  if (CHATTY_IS_PP_CHAT (self->chat) &&
+      chatty_pp_chat_run_command (CHATTY_PP_CHAT (self->chat), message)) {
     gtk_widget_hide (self->send_message_button);
     gtk_text_buffer_delete (self->message_input_buffer, &start, &end);
 
@@ -669,52 +415,29 @@ chat_view_send_message_button_clicked_cb (ChattyChatView *self)
 
   gtk_widget_grab_focus (self->message_input);
 
-  if (conv)
-    purple_idle_touch ();
-
-  message = gtk_text_buffer_get_text (self->message_input_buffer, &start, &end, FALSE);
-
   if (gtk_text_buffer_get_char_count (self->message_input_buffer)) {
     g_autofree char *escaped = NULL;
     ChattyProtocol protocol;
 
     protocol = chatty_item_get_protocols (CHATTY_ITEM (self->chat));
 
-    /* provide a msg-id to the sms-plugin for send-receipts */
-    if (chatty_item_get_protocols (CHATTY_ITEM (self->chat)) == CHATTY_PROTOCOL_SMS) {
-      sms_id = g_random_int ();
-
-      sms_id_str = g_strdup_printf ("%i", sms_id);
-
-      g_hash_table_insert (ht_sms_id, sms_id_str, g_object_ref (self->chat));
-
-      g_debug ("hash table insert sms_id_str: %s  ht_size: %i",
-               sms_id_str, g_hash_table_size (ht_sms_id));
-
-      purple_conv_im_send_with_flags (PURPLE_CONV_IM (conv),
-                                      sms_id_str,
-                                      PURPLE_MESSAGE_NO_LOG |
-                                      PURPLE_MESSAGE_NOTIFY |
-                                      PURPLE_MESSAGE_INVISIBLE);
-    }
-
     if (protocol == CHATTY_PROTOCOL_MATRIX ||
         protocol == CHATTY_PROTOCOL_XMPP ||
         protocol == CHATTY_PROTOCOL_TELEGRAM)
       escaped = purple_markup_escape_text (message, -1);
 
-    if (conv && purple_conversation_get_type (conv) == PURPLE_CONV_TYPE_IM) {
-      purple_conv_im_send (PURPLE_CONV_IM(conv), escaped ? escaped : message);
-    } else if (conv && purple_conversation_get_type (conv) == PURPLE_CONV_TYPE_CHAT) {
-      purple_conv_chat_send (PURPLE_CONV_CHAT (conv), escaped ? escaped : message);
-    }
+    msg = chatty_message_new (NULL, escaped ? escaped : message,
+                              NULL, time (NULL),
+                              escaped ? CHATTY_MESSAGE_HTML_ESCAPED : CHATTY_MESSAGE_TEXT,
+                              CHATTY_DIRECTION_OUT, 0);
+    chatty_chat_send_message_async (self->chat, msg,
+                                    view_send_message_async_cb,
+                                    g_object_ref (self));
 
     gtk_widget_hide (self->send_message_button);
   }
 
   gtk_text_buffer_delete (self->message_input_buffer, &start, &end);
-
-  g_free (message);
 }
 
 static gboolean
@@ -751,10 +474,11 @@ chat_view_message_input_changed_cb (ChattyChatView *self)
     chatty_update_typing_status (self);
 
   if (chatty_settings_get_convert_emoticons (chatty_settings_get_default ()) &&
-      chatty_item_get_protocols (CHATTY_ITEM (self->chat)) != CHATTY_PROTOCOL_SMS)
+      chatty_item_get_protocols (CHATTY_ITEM (self->chat)) != CHATTY_PROTOCOL_MMS_SMS)
     chatty_check_for_emoticon (self);
 }
 
+static void chat_view_adjustment_value_changed_cb (ChattyChatView *self);
 static void
 list_page_size_changed_cb (ChattyChatView *self)
 {
@@ -774,6 +498,20 @@ list_page_size_changed_cb (ChattyChatView *self)
     gtk_adjustment_set_value (self->vadjustment, upper);
 
   self->first_scroll_to_bottom = TRUE;
+  chat_view_adjustment_value_changed_cb (self);
+}
+
+static void
+chat_view_adjustment_value_changed_cb (ChattyChatView *self)
+{
+  gdouble value, upper, page_size;
+
+  upper = gtk_adjustment_get_upper (self->vadjustment);
+  value = gtk_adjustment_get_value (self->vadjustment);
+  page_size = gtk_adjustment_get_page_size (self->vadjustment);
+
+  gtk_widget_set_visible (self->scroll_down_button,
+                          (upper - value) > page_size + 1.0);
 }
 
 static void
@@ -804,56 +542,57 @@ chat_view_adjustment_changed_cb (GtkAdjustment  *adjustment,
     gtk_widget_set_visible (vscroll, FALSE);
     gtk_widget_show (self->encrypt_icon);
   }
+
+  chat_view_adjustment_value_changed_cb (self);
 }
 
 static void
-chat_view_sms_sent_cb (const char *sms_id,
-                       int         status)
+chat_view_update_header_func (ChattyMessageRow *row,
+                              ChattyMessageRow *before,
+                              gpointer          user_data)
 {
-  ChattyChat    *chat;
-  ChattyMessage *message;
-  GListModel    *message_list;
-  const gchar *message_id;
-  ChattyMsgStatus sent_status;
-  time_t       time_now;
-  guint        n_items;
+  ChattyChatView *self = user_data;
+  ChattyMessage *a, *b;
+  time_t a_time, b_time;
 
-  if (sms_id == NULL)
+  if (!before || !row)
     return;
 
-  if (status == CHATTY_SMS_RECEIPT_NONE)
-    sent_status = CHATTY_STATUS_SENDING_FAILED;
-  else if (status == CHATTY_SMS_RECEIPT_MM_ACKN)
-    sent_status = CHATTY_STATUS_SENT;
-  else if (status == CHATTY_SMS_RECEIPT_SMSC_ACKN)
-    sent_status = CHATTY_STATUS_DELIVERED;
-  else
+  a = chatty_message_row_get_item (before);
+  b = chatty_message_row_get_item (row);
+  a_time = chatty_message_get_time (a);
+  b_time = chatty_message_get_time (b);
+
+  if (chatty_message_user_matches (a, b))
+    chatty_message_row_hide_user_detail (row);
+
+  /* Don't hide footers in outgoing SMS as it helps understanding
+   * the delivery status of the message
+   */
+  if (CHATTY_IS_MM_CHAT (self->chat) &&
+      chatty_message_get_msg_direction (a) == CHATTY_DIRECTION_OUT)
     return;
 
-  chat = g_hash_table_lookup (ht_sms_id, sms_id);
+  /* Hide footer of the previous message if both have same time (in minutes) */
+  if (a_time / 60 == b_time / 60)
+    chatty_message_row_hide_footer (before);
+}
 
-  if (!chat)
-    return;
+static void
+chat_view_get_files_cb (GObject      *object,
+                        GAsyncResult *result,
+                        gpointer      user_data)
+{
+  g_autoptr(ChattyChatView) self = user_data;
+}
 
-  message_list = chatty_chat_get_messages (chat);
-  n_items = g_list_model_get_n_items (message_list);
-  message = g_list_model_get_item (message_list, n_items - 1);
-  message_id = chatty_message_get_id (message);
-  time_now = time (NULL);
-
-  if (message_id == NULL)
-    chatty_message_set_id (message, sms_id);
-
-  if (g_strcmp0 (message_id, sms_id) == 0) {
-    chatty_message_set_status (message, sent_status, time_now);
-    g_object_unref (message);
-    return;
-  }
-
-  message = chatty_pp_chat_find_message_with_id (CHATTY_PP_CHAT (chat), sms_id);
-
-  if (message)
-    chatty_message_set_status (message, sent_status, time_now);
+static void
+chat_view_file_requested_cb (ChattyChatView *self,
+                             ChattyMessage  *message)
+{
+  chatty_chat_get_files_async (self->chat, message,
+                               chat_view_get_files_cb,
+                               g_object_ref (self));
 }
 
 static void
@@ -871,9 +610,6 @@ chatty_chat_view_finalize (GObject *object)
 {
   ChattyChatView *self = (ChattyChatView *)object;
 
-  g_hash_table_foreach_remove (ht_sms_id,
-                               chat_view_hash_table_match_item,
-                               self);
   g_clear_object (&self->chat);
 
   G_OBJECT_CLASS (chatty_chat_view_parent_class)->finalize (object);
@@ -889,11 +625,20 @@ chatty_chat_view_class_init (ChattyChatViewClass *klass)
 
   widget_class->map = chatty_chat_view_map;
 
+  signals [FILE_REQUESTED] =
+    g_signal_new ("file-requested",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 1, CHATTY_TYPE_MESSAGE);
+
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/sm/puri/Chatty/"
                                                "ui/chatty-chat-view.ui");
 
+  gtk_widget_class_bind_template_child (widget_class, ChattyChatView, scroll_down_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, message_list);
+  gtk_widget_class_bind_template_child (widget_class, ChattyChatView, loading_spinner);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, typing_revealer);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, typing_indicator);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, input_frame);
@@ -905,10 +650,10 @@ chatty_chat_view_class_init (ChattyChatViewClass *klass)
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, empty_view);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, empty_label0);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, empty_label1);
-  gtk_widget_class_bind_template_child (widget_class, ChattyChatView, empty_label2);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, message_input_buffer);
   gtk_widget_class_bind_template_child (widget_class, ChattyChatView, vadjustment);
 
+  gtk_widget_class_bind_template_callback (widget_class, chat_view_scroll_down_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, chat_view_edge_overshot_cb);
   gtk_widget_class_bind_template_callback (widget_class, chat_view_typing_indicator_draw_cb);
   gtk_widget_class_bind_template_callback (widget_class, chat_view_input_focus_in_cb);
@@ -918,11 +663,13 @@ chatty_chat_view_class_init (ChattyChatViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, chat_view_input_key_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, chat_view_message_input_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, list_page_size_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, chat_view_adjustment_value_changed_cb);
 }
 
 static void
 chatty_chat_view_init (ChattyChatView *self)
 {
+  GspellTextView *gspell_view;
   GtkAdjustment *vadjustment;
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -932,6 +679,14 @@ chatty_chat_view_init (ChattyChatView *self)
   g_signal_connect_after (G_OBJECT (vadjustment), "notify::upper",
                           G_CALLBACK (chat_view_adjustment_changed_cb),
                           self);
+  g_signal_connect_after (G_OBJECT (self), "file-requested",
+                          G_CALLBACK (chat_view_file_requested_cb), self);
+  gtk_list_box_set_header_func (GTK_LIST_BOX (self->message_list),
+                                (GtkListBoxUpdateHeaderFunc)chat_view_update_header_func,
+                                g_object_ref (self), g_object_unref);
+
+  gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (self->message_input));
+  gspell_text_view_basic_setup (gspell_view);
 }
 
 GtkWidget *
@@ -940,62 +695,75 @@ chatty_chat_view_new (void)
   return g_object_new (CHATTY_TYPE_CHAT_VIEW, NULL);
 }
 
-
-void
-chatty_chat_view_purple_init (void)
-{
-  ht_sms_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
-
-  purple_signal_connect (purple_conversations_get_handle (),
-                         "sms-sent", ht_sms_id,
-                         PURPLE_CALLBACK (chat_view_sms_sent_cb), NULL);
-}
-
-void
-chatty_chat_view_purple_uninit (void)
-{
-  purple_signals_disconnect_by_handle (ht_sms_id);
-
-  g_hash_table_destroy (ht_sms_id);
-}
-
 void
 chatty_chat_view_set_chat (ChattyChatView *self,
                            ChattyChat     *chat)
 {
+  ChattyAccount *account;
+  GListModel *messages;
+
   g_return_if_fail (CHATTY_IS_CHAT_VIEW (self));
-  g_return_if_fail (CHATTY_IS_CHAT (chat));
+  g_return_if_fail (!chat || CHATTY_IS_CHAT (chat));
+
+  if (self->chat && chat != self->chat) {
+    g_signal_handlers_disconnect_by_func (self->chat,
+                                          chat_encrypt_changed_cb,
+                                          self);
+    g_signal_handlers_disconnect_by_func (self->chat,
+                                          chat_buddy_typing_changed_cb,
+                                          self);
+
+    g_clear_object (&self->history_binding);
+  }
+
+  gtk_widget_set_sensitive (self->message_input, !!chat);
+  gtk_widget_set_visible (self->empty_view, !!chat);
+  gtk_widget_set_visible (self->empty_view, !!chat);
 
   if (!g_set_object (&self->chat, chat))
     return;
 
-  if (!chat)
+  if (!chat) {
+    gtk_list_box_bind_model (GTK_LIST_BOX (self->message_list),
+                             NULL, NULL, NULL, NULL);
     return;
+  }
 
-  if (CHATTY_IS_PP_CHAT (chat))
-    self->conv = chatty_pp_chat_get_purple_conv (CHATTY_PP_CHAT (chat));
+  messages = chatty_chat_get_messages (chat);
+  account = chatty_chat_get_account (chat);
+
+  if (g_list_model_get_n_items (messages) <= 3)
+    chatty_chat_load_past_messages (chat, -1);
+
+
+  if (account)
+    g_signal_connect_object (account, "notify::status",
+                             G_CALLBACK (chat_account_status_changed_cb),
+                             self,
+                             G_CONNECT_SWAPPED);
+
+  chat_account_status_changed_cb (self);
 
   gtk_list_box_bind_model (GTK_LIST_BOX (self->message_list),
                            chatty_chat_get_messages (self->chat),
                            (GtkListBoxCreateWidgetFunc)chat_view_message_row_new,
                            self, NULL);
-  g_signal_connect_object (chatty_chat_get_messages (self->chat),
-                           "items-changed",
-                           G_CALLBACK (messages_items_changed_cb),
-                           self,
-                           G_CONNECT_SWAPPED | G_CONNECT_AFTER);
-  g_signal_connect_object (self->chat, "notify::encrypt",
-                           G_CALLBACK (chat_encrypt_changed_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
-  g_signal_connect_object (self->chat, "notify::buddy-typing",
-                           G_CALLBACK (chat_buddy_typing_changed_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
+  g_signal_connect_swapped (self->chat, "notify::encrypt",
+                            G_CALLBACK (chat_encrypt_changed_cb),
+                            self);
+  g_signal_connect_swapped (self->chat, "notify::buddy-typing",
+                            G_CALLBACK (chat_buddy_typing_changed_cb),
+                            self);
+  self->history_binding = g_object_bind_property (self->chat, "loading-history",
+                                                  self->loading_spinner, "active",
+                                                  G_BINDING_SYNC_CREATE);
 
   chat_encrypt_changed_cb (self);
   chat_buddy_typing_changed_cb (self);
   chatty_chat_view_update (self);
+  chat_view_adjustment_value_changed_cb (self);
+
+  gtk_widget_grab_focus (self->message_input);
 }
 
 ChattyChat *
