@@ -115,10 +115,9 @@ chatty_eds_bus_got (GObject      *object,
 }
 
 static ChattyContact *
-chatty_contact_provider_matches (ChattyEds      *self,
-                                 const char     *needle,
-                                 ChattyProtocol  protocols,
-                                 gboolean        match_name)
+chatty_contact_provider_get_match (ChattyEds      *self,
+                                   const char     *value,
+                                   ChattyProtocol  protocols)
 {
   GListModel *model;
   guint n_items;
@@ -129,13 +128,13 @@ chatty_contact_provider_matches (ChattyEds      *self,
 
   for (guint i = 0; i < n_items; i++)
     {
-      g_autoptr(ChattyItem) item = NULL;
+      g_autoptr(ChattyContact) contact = NULL;
 
-      item = g_list_model_get_item (model, i);
-      match = chatty_item_matches (item, needle, protocols, match_name);
+      contact = g_list_model_get_item (model, i);
+      match = chatty_contact_is_exact_match (contact, value, protocols);
 
       if (match)
-        return CHATTY_CONTACT (item);
+        return contact;
     }
 
   return NULL;
@@ -191,13 +190,14 @@ chatty_eds_load_contact (ChattyEds     *self,
     if (self->protocols & CHATTY_PROTOCOL_CALL)
       protocol = CHATTY_PROTOCOL_CALL;
     else
-      protocol = CHATTY_PROTOCOL_SMS;
+      protocol = CHATTY_PROTOCOL_MMS_SMS;
   } else if (field_id == E_CONTACT_IM_JABBER) {
     protocol = CHATTY_PROTOCOL_XMPP;
   } else {
     g_warn_if_reached ();
   }
 
+  /* Only container should be freed, attribute is freed in ChattyContact */
   attributes = e_contact_get_attributes (contact, field_id);
 
   for (GSList *l = (GSList *)attributes; l != NULL; l = l->next) {
@@ -239,7 +239,7 @@ chatty_eds_objects_added_cb (ChattyEds       *self,
 
   for (GSList *l = (GSList *)objects; l != NULL; l = l->next)
     {
-      if (self->protocols & CHATTY_PROTOCOL_SMS ||
+      if (self->protocols & CHATTY_PROTOCOL_MMS_SMS ||
           self->protocols & CHATTY_PROTOCOL_CALL)
         chatty_eds_load_contact (self, l->data, E_CONTACT_TEL);
 
@@ -364,6 +364,7 @@ chatty_eds_client_connected_cb (GObject      *object,
     {
       if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("Error: %s", error->message);
+      g_clear_object (&client);
 
       return;
     }
@@ -378,6 +379,7 @@ chatty_eds_client_connected_cb (GObject      *object,
                           NULL,
                           chatty_eds_get_view_cb,
                           g_object_ref (self));
+  g_clear_object (&client);
 }
 
 
@@ -495,6 +497,10 @@ chatty_eds_finalize (GObject *object)
 
   g_cancellable_cancel (self->cancellable);
   g_clear_object (&self->cancellable);
+  g_clear_object (&self->eds_view_list);
+  g_clear_object (&self->contacts_list);
+  if (self->contacts_array)
+    g_ptr_array_free (self->contacts_array, TRUE);
 
   G_OBJECT_CLASS (chatty_eds_parent_class)->finalize (object);
 }
@@ -634,7 +640,7 @@ chatty_eds_find_by_number (ChattyEds  *self,
 {
   g_return_val_if_fail (CHATTY_IS_EDS (self), NULL);
 
-  return chatty_contact_provider_matches (self, phone_number, CHATTY_PROTOCOL_ANY, FALSE);
+  return chatty_contact_provider_get_match (self, phone_number, CHATTY_PROTOCOL_ANY);
 }
 
 
@@ -672,3 +678,142 @@ chatty_eds_open_contacts_app_finish (ChattyEds    *self,
 
   return g_task_propagate_boolean (G_TASK (result), error);
 }
+
+static void
+chatty_save_contact_cb (GObject      *object,
+                        GAsyncResult *result,
+                        gpointer      user_data)
+{
+  GDBusConnection *connection = (GDBusConnection *)object;
+  g_autoptr(GTask) task = user_data;
+  g_autoptr(GVariant) variant = NULL;
+  GError *error = NULL;
+
+  g_assert (G_IS_DBUS_CONNECTION (connection));
+  g_assert (G_IS_TASK (task));
+
+  variant = g_dbus_connection_call_finish (connection, result, &error);
+
+  if (error) {
+    g_dbus_error_strip_remote_error (error);
+    g_warning ("Error saving to contacts: %s", error->message);
+    g_task_return_error (task, error);
+  } else {
+    g_task_return_boolean (task, TRUE);
+  }
+}
+
+static void
+chatty_check_save_contact_cb (GObject      *object,
+                              GAsyncResult *result,
+                              gpointer      user_data)
+{
+  GDBusConnection *connection = (GDBusConnection *)object;
+  g_autoptr(GTask) task = user_data;
+  g_autoptr(GVariant) variant = NULL;
+  GError *error = NULL;
+
+  g_assert (G_IS_DBUS_CONNECTION (connection));
+  g_assert (G_IS_TASK (task));
+
+  variant = g_dbus_connection_call_finish (connection, result, &error);
+
+  if (error) {
+    g_dbus_error_strip_remote_error (error);
+    g_warning ("Error saving to contacts: %s", error->message);
+    g_task_return_error (task, error);
+  } else {
+    GCancellable *cancellable;
+    GVariantBuilder options;
+    GVariant *contact;
+
+    cancellable = g_task_get_cancellable (task);
+    contact = g_task_get_task_data (task);
+    g_assert (contact);
+
+    g_variant_builder_init (&options, G_VARIANT_TYPE ("av"));
+    g_variant_builder_add (&options, "v", contact);
+
+    g_dbus_connection_call (connection,
+                            "org.gnome.Contacts",
+                            "/org/gnome/Contacts",
+                            "org.gtk.Actions",
+                            "Activate",
+                            g_variant_new ("(sava{sv})",
+                                           "new-contact-data", &options, NULL),
+                            NULL,
+                            G_DBUS_CALL_FLAGS_NONE,
+                            -1,
+                            cancellable,
+                            chatty_save_contact_cb,
+                            g_steal_pointer (&task));
+  }
+}
+
+static void
+eds_write_contact_bus_got (GObject      *object,
+                           GAsyncResult *result,
+                           gpointer      user_data)
+{
+  g_autoptr(GTask) task = user_data;
+  g_autoptr(GDBusConnection) connection = NULL;
+  GCancellable *cancellable;
+  GError *error = NULL;
+
+  g_assert (G_IS_TASK (task));
+
+  connection  = g_bus_get_finish (result, &error);
+  cancellable = g_task_get_cancellable (task);
+
+  /* XXX: A direct call to 'new-contact-data' action doesn't seem to fail
+     if the action is missing, So let's manually check if it exists */
+  if (error) {
+    g_warning ("Error getting dbus: %s", error->message);
+    g_task_return_error (task, error);
+  } else {
+    g_dbus_connection_call (connection,
+                            "org.gnome.Contacts",
+                            "/org/gnome/Contacts",
+                            "org.gtk.Actions",
+                            "Describe",
+                            g_variant_new ("(s)", "new-contact-data"),
+                            NULL,
+                            G_DBUS_CALL_FLAGS_NONE,
+                            -1,
+                            cancellable,
+                            chatty_check_save_contact_cb,
+                            g_steal_pointer (&task));
+  }
+}
+
+void
+chatty_eds_write_contact_async (const char          *name,
+                                const char          *phone_number,
+                                GAsyncReadyCallback  callback,
+                                gpointer             user_data)
+{
+  g_autoptr(GTask) task = NULL;
+  GVariant *contact;
+
+  g_return_if_fail (name);
+  g_return_if_fail (phone_number);
+
+  contact =  g_variant_new_parsed ("[('full-name', %s), ('phone-numbers', %s)]",
+                                   name, phone_number);
+
+  task = g_task_new (NULL, NULL, callback, user_data);
+  g_task_set_task_data (task, contact, NULL);
+
+  g_bus_get (G_BUS_TYPE_SESSION,
+             NULL,
+             eds_write_contact_bus_got,
+             g_steal_pointer (&task));
+}
+
+gboolean
+chatty_eds_write_contact_finish (GAsyncResult  *result,
+                                 GError       **error)
+{
+  return g_task_propagate_boolean (G_TASK (result), error);
+}
+
