@@ -19,6 +19,7 @@
  */
 typedef enum
 {
+  CHATTY_UNKNOWN,
   CHATTY_DISCONNECTED,
   CHATTY_CONNECTING,
   CHATTY_CONNECTED,
@@ -48,7 +49,7 @@ typedef enum
 typedef enum
 {
   CHATTY_PROTOCOL_NONE     = 0,
-  CHATTY_PROTOCOL_SMS      = 1 << 0,
+  CHATTY_PROTOCOL_MMS_SMS  = 1 << 0,
   CHATTY_PROTOCOL_MMS      = 1 << 1,
   CHATTY_PROTOCOL_CALL     = 1 << 2,
   CHATTY_PROTOCOL_XMPP     = 1 << 3,
@@ -128,5 +129,23 @@ typedef enum
   CHATTY_MESSAGE_FILE,
   CHATTY_MESSAGE_IMAGE,
   CHATTY_MESSAGE_VIDEO,
-  chatty_message_AUDIO,
+  CHATTY_MESSAGE_AUDIO,
 } ChattyMsgType;
+
+typedef enum
+{
+  CHATTY_FILE_UNKNOWN,
+  CHATTY_FILE_DOWNLOADED = 1,
+  CHATTY_FILE_MISSING,
+  CHATTY_FILE_DECRYPT_FAILED,
+  CHATTY_FILE_DOWNLOADING,
+  CHATTY_FILE_ERROR,
+} ChattyFileStatus;
+
+typedef enum
+{
+  CHATTY_ITEM_VISIBLE,
+  CHATTY_ITEM_HIDDEN,
+  CHATTY_ITEM_ARCHIVED,
+  CHATTY_ITEM_BLOCKED,
+} ChattyItemState;
