@@ -26,8 +26,6 @@ GtkWidget     *chatty_message_row_new              (ChattyMessage  *message,
                                                     ChattyProtocol  protocol,
                                                     gboolean        is_im);
 ChattyMessage *chatty_message_row_get_item         (ChattyMessageRow *self);
-void           chatty_message_row_set_footer       (ChattyMessageRow *self,
-                                                    GtkWidget        *footer);
 void           chatty_message_row_hide_footer      (ChattyMessageRow *self);
 void           chatty_message_row_set_alias        (ChattyMessageRow *self,
                                                     const char       *alias);
