@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
+#define G_LOG_DOMAIN "chatty-utils"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -14,6 +15,7 @@
 #include <libebook-contacts/libebook-contacts.h>
 #include <gdesktop-enums.h>
 
+#include "chatty-log.h"
 
 #define DIGITS      "0123456789"
 #define ASCII_CAPS  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -80,7 +82,7 @@ chatty_utils_check_phonenumber (const char *phone_number,
   char              *result;
   g_autoptr(GError)  err = NULL;
 
-  g_debug ("%s number %s", G_STRLOC, phone_number);
+  CHATTY_DEBUG (phone_number, "checking number");
 
   if (!phone_number || !*phone_number)
     return NULL;
@@ -186,7 +188,7 @@ chatty_utils_username_is_valid (const char     *name,
       valid |= CHATTY_PROTOCOL_TELEGRAM;
   }
 
-  if (protocol & CHATTY_PROTOCOL_SMS && len < 20) {
+  if (protocol & CHATTY_PROTOCOL_MMS_SMS && len < 20) {
     const char *end;
     guint end_len;
 
@@ -200,7 +202,7 @@ chatty_utils_username_is_valid (const char     *name,
       end_len++;
 
     if (end_len == len)
-      valid |= CHATTY_PROTOCOL_SMS;
+      valid |= CHATTY_PROTOCOL_MMS_SMS;
   }
 
   return valid;
@@ -334,97 +336,6 @@ chatty_utils_remove_list_item (GListStore *store,
   return FALSE;
 }
 
-GtkWidget*
-chatty_utils_create_fingerprint_row (const char *fp,
-                                     guint       id)
-{
-  GtkWidget     *row;
-  GtkBox        *vbox;
-  GtkLabel      *label_fp;
-  GtkLabel      *label_id;
-  g_auto(GStrv)  line_split = NULL;
-  char          *markup_fp = NULL;
-  char          *markup_id = NULL;
-  char          *device_id;
-
-  if (!fp) {
-    return NULL;
-  }
-
-  line_split = g_strsplit (fp, " ", -1);
-
-  markup_fp = "<span font_family='monospace' font='9'>";
-
-  for (int i = 0; i < 8; i++) {
-    markup_fp = g_strconcat (markup_fp,
-                             i % 2 ? "<span color='DarkGrey'>"
-                                   : "<span color='DimGrey'>",
-                             line_split[i],
-                             i == 3 ? "\n" : " ",
-                             "</span>",
-                             i == 7 ? "</span>" : "\0",
-                             NULL);
-  }
-
-  device_id = g_strdup_printf ("%i", id);
-
-  markup_id = g_strconcat ("<span font='9'>"
-                           "<span color='DimGrey'>",
-                           "Device ID ",
-                           device_id,
-                           " fingerprint:",
-                           "</span></span>",
-                           NULL);
-
-  row = GTK_WIDGET(gtk_list_box_row_new ());
-  g_object_set (G_OBJECT(row),
-                "selectable", FALSE,
-                "activatable", FALSE,
-                NULL);
-
-  vbox = GTK_BOX(gtk_box_new (GTK_ORIENTATION_VERTICAL, 0));
-  g_object_set (G_OBJECT(vbox),
-                "margin_top", 6,
-                "margin_bottom", 6,
-                "margin_start", 12,
-                "margin_end", 6,
-                NULL);
-
-  label_id = GTK_LABEL(gtk_label_new (NULL));
-  gtk_label_set_markup (GTK_LABEL(label_id), g_strdup (markup_id));
-  g_object_set (G_OBJECT(label_id),
-                "can_focus", FALSE,
-                "use_markup", TRUE,
-                "ellipsize", PANGO_ELLIPSIZE_END,
-                "halign", GTK_ALIGN_START,
-                "hexpand", TRUE,
-                "xalign", 0.0,
-                NULL);
-
-  label_fp = GTK_LABEL(gtk_label_new (NULL));
-  gtk_label_set_markup (GTK_LABEL(label_fp), g_strdup (markup_fp));
-  g_object_set (G_OBJECT(label_fp),
-                "can_focus", FALSE,
-                "use_markup", TRUE,
-                "ellipsize", PANGO_ELLIPSIZE_END,
-                "halign", GTK_ALIGN_START,
-                "hexpand", TRUE,
-                "margin_top", 8,
-                "xalign", 0.0,
-                NULL);
-
-  gtk_box_pack_start (vbox, GTK_WIDGET(label_id), FALSE, FALSE, 0);
-  gtk_box_pack_start (vbox, GTK_WIDGET(label_fp), FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER(row), GTK_WIDGET(vbox));
-  gtk_widget_show_all (GTK_WIDGET(row));
-
-  g_free (markup_fp);
-  g_free (markup_id);
-  g_free (device_id);
-
-  return GTK_WIDGET(row);
-}
-
 char *
 chatty_utils_get_human_time (time_t unix_time)
 {
@@ -506,6 +417,39 @@ chatty_utils_get_conv_blist_node (PurpleConversation *conv)
   return node;
 }
 
+GdkPixbuf *
+chatty_utils_get_pixbuf_from_data (const guchar *buf,
+                                   gsize         count)
+{
+  g_autoptr(GdkPixbufLoader) loader = NULL;
+  g_autoptr(GError) error = NULL;
+  GdkPixbuf *pixbuf;
+
+  if (!buf || !count)
+    return NULL;
+
+  loader = gdk_pixbuf_loader_new ();
+  gdk_pixbuf_loader_write (loader, buf, count, &error);
+
+  if (!error)
+    gdk_pixbuf_loader_close (loader, &error);
+
+  if (error) {
+    CHATTY_TRACE_MSG ("%s", error->message);
+    return NULL;
+  }
+
+  pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+
+  if (!pixbuf) {
+    CHATTY_TRACE_MSG ("pixbuf creation failed");
+    return NULL;
+  }
+
+  return g_object_ref (pixbuf);
+}
+
+
 ChattyMsgDirection
 chatty_utils_direction_from_flag (PurpleMessageFlags flag)
 {
@@ -520,3 +464,16 @@ chatty_utils_direction_from_flag (PurpleMessageFlags flag)
 
   g_return_val_if_reached (CHATTY_DIRECTION_UNKNOWN);
 }
+
+void
+chatty_file_info_free (ChattyFileInfo *file_info)
+{
+  if (!file_info)
+    return;
+
+  g_free (file_info->file_name);
+  g_free (file_info->url);
+  g_free (file_info->path);
+  g_free (file_info->mime_type);
+  g_free (file_info);
+}
