@@ -11,12 +11,29 @@
 #include <stdio.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
+#include <purple.h>
 
-#include "users/chatty-pp-account.h"
+#include "chatty-enums.h"
 
 #define MAX_GMT_ISO_SIZE 256
 #define SECONDS_PER_DAY    86400.0
 
+typedef struct _ChattyFileInfo ChattyFileInfo;
+
+struct _ChattyFileInfo {
+  char *file_name;
+  char *url;
+  char *path;
+  char *mime_type;
+  gpointer user_data;
+  gsize width;
+  gsize height;
+  gsize size;
+  /* For audio files */
+  gsize duration;
+  int status;
+};
+
 char *chatty_utils_jabber_id_strip (const char *name);
 char *chatty_utils_check_phonenumber (const char *phone_number,
                                       const char *country);
@@ -29,9 +46,11 @@ gboolean chatty_utils_get_item_position (GListModel *list,
                                          guint      *position);
 gboolean chatty_utils_remove_list_item  (GListStore *store,
                                          gpointer    item);
-GtkWidget* chatty_utils_create_fingerprint_row (const char *fp, guint id);
 char       *chatty_utils_get_human_time (time_t unix_time);
 PurpleBlistNode *chatty_utils_get_conv_blist_node (PurpleConversation *conv);
+GdkPixbuf           *chatty_utils_get_pixbuf_from_data  (const guchar *buf,
+                                                         gsize         count);
 ChattyMsgDirection   chatty_utils_direction_from_flag   (PurpleMessageFlags flag);
+void                 chatty_file_info_free              (ChattyFileInfo     *file_info);
 
 #endif
