@@ -4,34 +4,37 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
+#define G_LOG_DOMAIN "chatty-window"
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #define _GNU_SOURCE
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
-#include <purple.h>
 #include "contrib/gtk.h"
-#include "chatty-config.h"
+
 #include "chatty-window.h"
-#include "chatty-dbus.h"
 #include "chatty-history.h"
 #include "chatty-avatar.h"
 #include "chatty-manager.h"
 #include "chatty-list-row.h"
 #include "chatty-settings.h"
+#include "chatty-mm-chat.h"
 #include "chatty-pp-chat.h"
 #include "chatty-chat-view.h"
 #include "chatty-manager.h"
-#include "chatty-icons.h"
 #include "chatty-utils.h"
+#include "matrix/chatty-ma-account.h"
+#include "matrix/chatty-ma-chat.h"
+#include "dialogs/chatty-info-dialog.h"
 #include "dialogs/chatty-settings-dialog.h"
 #include "dialogs/chatty-new-chat-dialog.h"
 #include "dialogs/chatty-new-muc-dialog.h"
-#include "dialogs/chatty-user-info-dialog.h"
-#include "dialogs/chatty-muc-info-dialog.h"
-
-#define LAZY_LOAD_INITIAL_MSGS_LIMIT 20
+#include "chatty-log.h"
 
 struct _ChattyWindow
 {
@@ -52,6 +55,7 @@ struct _ChattyWindow
   GtkWidget *sub_header_label;
 
   GtkWidget *new_chat_dialog;
+  GtkWidget *chat_info_dialog;
 
   GtkWidget *search_button;
   GtkWidget *chats_search_bar;
@@ -60,21 +64,15 @@ struct _ChattyWindow
   GtkWidget *header_chat_list_new_msg_popover;
 
   GtkWidget *menu_add_contact_button;
-  GtkWidget *menu_add_in_contacts_button;
   GtkWidget *menu_new_message_button;
   GtkWidget *menu_new_group_message_button;
-  GtkWidget *menu_new_bulk_sms_button;
-  GtkWidget *header_chat_info_button;
   GtkWidget *header_add_chat_button;
+  GtkWidget *call_button;
   GtkWidget *header_sub_menu_button;
+  GtkWidget *leave_button;
+  GtkWidget *delete_button;
 
-  GtkWidget *convs_notebook;
-
-  GtkWidget *overlay_icon;
-  GtkWidget *overlay_label_1;
-  GtkWidget *overlay_label_2;
-  GtkWidget *overlay_label_3;
-
+  GtkWidget *chat_view;
 
   ChattyItem    *selected_item;
   ChattyManager *manager;
@@ -87,77 +85,7 @@ struct _ChattyWindow
 
 G_DEFINE_TYPE (ChattyWindow, chatty_window, GTK_TYPE_APPLICATION_WINDOW)
 
-
-static void chatty_window_show_new_muc_dialog (ChattyWindow *self);
-
-
-typedef struct {
-  const char *title;
-  const char *text_1;
-  const char *text_2;
-  const char *icon_name;
-  int         icon_size;
-} overlay_content_t;
-
-overlay_content_t OverlayContent[6] = {
-  {.title  = N_("Choose a contact"),
-   .text_1 = N_("Select an <b>SMS</b> or <b>Instant Message</b> "
-                "contact with the <b>\"+\"</b> button in the titlebar."),
-   .text_2 = NULL,
-  },
-  {.title  = N_("Choose a contact"),
-   .text_1 = N_("Select an <b>Instant Message</b> contact with "
-                "the \"+\" button in the titlebar."),
-   .text_2 = NULL,
-  },
-  {.title  = N_("Choose a contact"),
-   .text_1 = N_("Start a <b>SMS</b> chat with the \"+\" button in the titlebar."),
-   .text_2 = N_("For <b>Instant Messaging</b> add or activate "
-                "an account in <i>\"preferences\"</i>."),
-  },
-  {.title  = N_("Start chatting"),
-   .text_1 = N_("For <b>Instant Messaging</b> add or activate "
-                "an account in <i>\"preferences\"</i>."),
-   .text_2 = NULL,
-  }
-};
-
-
-static GtkWidget *
-window_chat_list_row_new (ChattyItem   *item,
-                          ChattyWindow *self)
-{
-  GtkWidget *row;
-
-  g_assert (CHATTY_IS_ITEM (item));
-  g_assert (CHATTY_IS_WINDOW (self));
-
-  row = chatty_list_row_new (item);
-
-  if (self->selected_item == item)
-    gtk_widget_set_state_flags (row, GTK_STATE_FLAG_SELECTED, FALSE);
-
-  return row;
-}
-
-static GtkWidget *
-window_get_view_for_chat (ChattyWindow *self,
-                          ChattyChat   *chat)
-{
-  g_autoptr(GList) children = NULL;
-
-  g_assert (CHATTY_IS_WINDOW (self));
-  g_assert (CHATTY_IS_CHAT (chat));
-
-  children = gtk_container_get_children (GTK_CONTAINER (self->convs_notebook));
-
-  for (GList *child = children; child; child = child->next)
-    if (CHATTY_IS_CHAT_VIEW (child->data) &&
-        chatty_chat_view_get_chat (child->data) == chat)
-      return child->data;
-
-  return NULL;
-}
+static void chatty_window_chat_list_select_first (ChattyWindow *self);
 
 static void
 window_set_item (ChattyWindow *self,
@@ -173,6 +101,9 @@ window_set_item (ChattyWindow *self,
   self->selected_item = item;
   chatty_avatar_set_item (CHATTY_AVATAR (self->sub_header_icon), item);
   gtk_label_set_label (GTK_LABEL (self->sub_header_label), header_label);
+
+  if (!item)
+    hdy_leaflet_set_visible_child_name (HDY_LEAFLET (self->content_box), "sidebar");
 }
 
 static void
@@ -184,7 +115,7 @@ chatty_window_update_sidebar_view (ChattyWindow *self)
 
   g_assert (CHATTY_IS_WINDOW (self));
 
-  model = chatty_manager_get_chat_list (self->manager);
+  model = G_LIST_MODEL (self->filter_model);
   has_child = g_list_model_get_n_items (model) > 0;
 
   if (has_child)
@@ -203,18 +134,20 @@ static void
 window_chat_changed_cb (ChattyWindow *self)
 {
   GListModel *model;
-  ChattyProtocol protocols;
-  gint mode = CHATTY_OVERLAY_EMPTY_CHAT_NO_SMS_IM;
-  gboolean has_sms, has_im;
   gboolean has_child;
 
   g_assert (CHATTY_IS_WINDOW (self));
 
-  model = chatty_manager_get_chat_list (self->manager);
+  model = G_LIST_MODEL (self->filter_model);
   has_child = g_list_model_get_n_items (model) > 0;
 
   gtk_widget_set_sensitive (self->header_sub_menu_button, !!self->selected_item);
 
+  /* Select the first item if none selected and we are in expanded mode */
+  if (!self->selected_item &&
+      gtk_list_box_get_selection_mode (GTK_LIST_BOX (self->chats_listbox)) == GTK_SELECTION_SINGLE)
+    chatty_window_chat_list_select_first (self);
+
   /*
    * When the items are re-arranged, the selection will be lost.
    * Re-select it.  In GTK4, A #GtkListView with #GtkSingleSelection
@@ -223,8 +156,6 @@ window_chat_changed_cb (ChattyWindow *self)
   if (self->selected_item && has_child) {
     guint position;
 
-    chatty_chat_set_unread_count (CHATTY_CHAT (self->selected_item), 0);
-
     if (chatty_utils_get_item_position (G_LIST_MODEL (self->filter_model), self->selected_item, &position)) {
       GtkListBoxRow *row;
 
@@ -238,63 +169,33 @@ window_chat_changed_cb (ChattyWindow *self)
   if (has_child)
     return;
 
-  protocols = chatty_manager_get_active_protocols (self->manager);
-  has_sms = !!(protocols & CHATTY_PROTOCOL_SMS);
-  has_im  = !!(protocols & ~CHATTY_PROTOCOL_SMS);
-
-  if (has_sms && has_im)
-    mode = CHATTY_OVERLAY_EMPTY_CHAT;
-  else if (has_sms)
-    mode = CHATTY_OVERLAY_EMPTY_CHAT_NO_IM;
-  else if (has_im)
-    mode = CHATTY_OVERLAY_EMPTY_CHAT_NO_SMS;
-
-  gtk_label_set_markup (GTK_LABEL (self->overlay_label_1),
-                        gettext (OverlayContent[mode].title));
-  gtk_label_set_markup (GTK_LABEL (self->overlay_label_2),
-                        gettext (OverlayContent[mode].text_1));
-  gtk_label_set_markup (GTK_LABEL (self->overlay_label_3),
-                        gettext (OverlayContent[mode].text_2));
+  if (chatty_manager_get_active_protocols (self->manager))
+    hdy_status_page_set_description (HDY_STATUS_PAGE (self->empty_view),
+                                     _("Select a contact with the "
+                                       "<b>“+”</b> button in the titlebar."));
+  else
+    hdy_status_page_set_description (HDY_STATUS_PAGE (self->empty_view),
+                                     _("For <b>Instant Messaging</b> add or activate "
+                                       "an account in <i>\"preferences\"</i>."));
 }
 
-static void
-window_notebook_after_switch_cb (GtkNotebook  *notebook,
-                                 GtkWidget    *page,
-                                 gint          page_num,
-                                 ChattyWindow *self)
-{
-  ChattyChat *chat;
-
-  g_assert (CHATTY_IS_CHAT_VIEW (page));
-
-  chat = chatty_chat_view_get_chat (CHATTY_CHAT_VIEW (page));
-  window_set_item (self, CHATTY_ITEM (chat));
-  window_chat_changed_cb (self);
-
-  g_debug ("%s: Chat name: %s", G_STRFUNC, chatty_chat_get_chat_name (chat));
-
-  chatty_chat_set_unread_count (chat, 0);
-  gtk_widget_set_visible (self->header_chat_info_button,
-                          !chatty_chat_is_im (chat));
-}
-
-
 static gboolean
 window_chat_name_matches (ChattyItem   *item,
                           ChattyWindow *self)
 {
-  ChattyProtocol protocols;
+  ChattyProtocol protocol;
 
   g_assert (CHATTY_IS_CHAT (item));
   g_assert (CHATTY_IS_WINDOW (self));
 
-  protocols = chatty_manager_get_active_protocols (self->manager);
+  protocol = chatty_item_get_protocols (item);
 
-  if (!(protocols & chatty_item_get_protocols (item)))
-    return FALSE;
+  if ((!self->chat_needle || !*self->chat_needle) &&
+      CHATTY_IS_MM_CHAT (item))
+    return TRUE;
 
   /* FIXME: Not a good idea */
-  if (chatty_item_get_protocols (item) != CHATTY_PROTOCOL_SMS) {
+  if (chatty_item_get_protocols (item) != CHATTY_PROTOCOL_MMS_SMS) {
     ChattyAccount *account;
 
     if (CHATTY_IS_PP_CHAT (item) &&
@@ -307,7 +208,8 @@ window_chat_name_matches (ChattyItem   *item,
       return FALSE;
   }
 
-  if (hdy_leaflet_get_folded (HDY_LEAFLET (self->header_box))) {
+  if (protocol != CHATTY_PROTOCOL_MATRIX &&
+      hdy_leaflet_get_folded (HDY_LEAFLET (self->header_box))) {
     GListModel *message_list;
     guint n_items;
 
@@ -331,66 +233,49 @@ static void
 chatty_window_open_item (ChattyWindow *self,
                          ChattyItem   *item)
 {
-  PurpleBlistNode *node = NULL;
-  PurpleChat *chat;
+  ChattyChat *chat = NULL;
 
   g_assert (CHATTY_IS_WINDOW (self));
   g_assert (CHATTY_IS_ITEM (item));
+  g_debug ("opening item of type: %s, protocol: %d",
+           G_OBJECT_TYPE_NAME (item),
+           chatty_item_get_protocols (item));
 
   if (CHATTY_IS_CONTACT (item)) {
     const char *number;
 
-    number = chatty_contact_get_value (CHATTY_CONTACT (item));
+    number = chatty_item_get_username (item);
     chatty_window_set_uri (self, number);
 
     return;
   }
 
-  if (CHATTY_IS_PP_BUDDY (item))
-    node = (PurpleBlistNode *)chatty_pp_buddy_get_buddy (CHATTY_PP_BUDDY (item));
+  if (CHATTY_IS_PP_BUDDY (item)) {
+    chat = g_object_get_data (G_OBJECT (item), "chat");
 
-  if (CHATTY_IS_PP_CHAT (item))
-    node = (PurpleBlistNode *)chatty_pp_chat_get_purple_buddy (CHATTY_PP_CHAT (item));
+    if (!chat) {
+      gboolean has_encryption;
 
-  if (node) {
-    PurpleAccount *account;
-    PurpleBuddy *buddy;
-
-    buddy = (PurpleBuddy*)node;
-    account = purple_buddy_get_account (buddy);
-
-    if (chatty_item_get_protocols (item) == CHATTY_PROTOCOL_SMS) {
-      ChattyEds *chatty_eds;
-      ChattyContact *contact;
-      const char *number;
-
-      chatty_eds = chatty_manager_get_eds (self->manager);
-      number = purple_buddy_get_name (buddy);
-      contact = chatty_eds_find_by_number (chatty_eds, number);
-
-      if (!contact)
-        gtk_widget_show (self->menu_add_in_contacts_button);
+      has_encryption = chatty_manager_lurch_plugin_is_loaded (self->manager);
+      chat = (ChattyChat *)chatty_pp_chat_new_buddy_chat (CHATTY_PP_BUDDY (item),
+                                                          has_encryption);
+      chatty_manager_add_chat (self->manager, chat);
     }
-
-    if (purple_blist_node_get_bool (node, "chatty-unknown-contact"))
-      gtk_widget_show (self->menu_add_contact_button);
-
-    purple_blist_node_set_bool (PURPLE_BLIST_NODE (buddy), "chatty-autojoin", TRUE);
-    chatty_conv_im_with_buddy (account, purple_buddy_get_name (buddy));
-
-    return;
   }
 
-  if (CHATTY_IS_PP_CHAT (item) &&
-      (chat = chatty_pp_chat_get_purple_chat (CHATTY_PP_CHAT (item)))) {
-    chatty_conv_join_chat (chat);
+  if (CHATTY_IS_PP_CHAT (item))
+    chat = CHATTY_CHAT (item);
 
-    purple_blist_node_set_bool ((PurpleBlistNode *)chat, "chatty-autojoin", TRUE);
-    chatty_window_change_view (self, CHATTY_VIEW_MESSAGE_LIST);
+  if (CHATTY_IS_PP_CHAT (chat)) {
+    chatty_pp_chat_join (CHATTY_PP_CHAT (chat));
 
     gtk_filter_changed (self->chat_filter, GTK_FILTER_CHANGE_DIFFERENT);
     window_chat_changed_cb (self);
   }
+
+  if (CHATTY_IS_MM_CHAT (item)) {
+    chatty_window_open_chat (CHATTY_WINDOW (self), CHATTY_CHAT (item));
+  }
 }
 
 static void
@@ -404,7 +289,27 @@ window_chat_row_activated_cb (GtkListBox    *box,
 
   g_return_if_fail (CHATTY_IS_CHAT (self->selected_item));
 
-  chatty_window_open_item (self, self->selected_item);
+  if (CHATTY_IS_PP_CHAT (self->selected_item))
+    chatty_window_open_item (self, self->selected_item);
+  else
+    chatty_window_open_chat (self, CHATTY_CHAT (self->selected_item));
+}
+
+static void
+window_call_button_clicked_cb (ChattyWindow *self)
+{
+  g_autoptr(GError) error = NULL;
+  g_autofree char *uri = NULL;
+
+  g_assert (CHATTY_IS_WINDOW (self));
+  g_return_if_fail (CHATTY_IS_MM_CHAT (self->selected_item));
+
+  uri = g_strconcat ("tel://",
+                     chatty_chat_get_chat_name (CHATTY_CHAT (self->selected_item)),
+                     NULL);
+
+  if (!gtk_show_uri_on_window (NULL, uri, GDK_CURRENT_TIME, &error))
+    g_warning ("Failed to launch call: %s", error->message);
 }
 
 static void
@@ -420,9 +325,7 @@ window_search_changed_cb (ChattyWindow *self,
 }
 
 static void
-notify_fold_cb (GObject      *sender,
-                GParamSpec   *pspec,
-                ChattyWindow *self)
+notify_fold_cb (ChattyWindow *self)
 {
   gboolean folded = hdy_leaflet_get_folded (HDY_LEAFLET (self->header_box));
 
@@ -433,7 +336,6 @@ notify_fold_cb (GObject      *sender,
 
   if (folded) {
     window_set_item (self, NULL);
-    hdy_leaflet_set_visible_child_name (HDY_LEAFLET (self->content_box), "sidebar");
   } else if (self->selected_item) {
     window_chat_changed_cb (self);
   } else {
@@ -460,6 +362,7 @@ window_new_message_clicked_cb (ChattyWindow *self)
   if (response != GTK_RESPONSE_OK)
     return;
 
+
   dialog = CHATTY_NEW_CHAT_DIALOG (self->new_chat_dialog);
   item = chatty_new_chat_dialog_get_selected_item (dialog);
 
@@ -468,7 +371,7 @@ window_new_message_clicked_cb (ChattyWindow *self)
 
   if (CHATTY_IS_CONTACT (item) &&
       chatty_contact_is_dummy (CHATTY_CONTACT (item)))
-    phone_number = chatty_contact_get_value (CHATTY_CONTACT (item));
+    phone_number = chatty_item_get_username (item);
 
   if (phone_number)
     chatty_window_set_uri (self, phone_number);
@@ -482,9 +385,14 @@ window_new_message_clicked_cb (ChattyWindow *self)
 static void
 window_new_muc_clicked_cb (ChattyWindow *self)
 {
+  GtkWidget *dialog;
+
   g_assert (CHATTY_IS_WINDOW (self));
 
-  chatty_window_show_new_muc_dialog (self);
+  dialog = chatty_new_muc_dialog_new (GTK_WINDOW (self));
+  gtk_dialog_run (GTK_DIALOG (dialog));
+
+  gtk_widget_destroy (dialog);
 }
 
 
@@ -493,7 +401,7 @@ window_add_chat_button_clicked_cb (ChattyWindow *self)
 {
   g_assert (CHATTY_IS_WINDOW (self));
 
-  if (chatty_manager_get_active_protocols (self->manager) == CHATTY_PROTOCOL_SMS)
+  if (chatty_manager_get_active_protocols (self->manager) == CHATTY_PROTOCOL_MMS_SMS)
     window_new_message_clicked_cb (self);
   else
     gtk_popover_popup (GTK_POPOVER (self->header_chat_list_new_msg_popover));
@@ -513,11 +421,9 @@ window_back_clicked_cb (ChattyWindow *self)
    * while chatting with this node
    */
   gtk_list_box_unselect_all (GTK_LIST_BOX (self->chats_listbox));
-  chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
 }
 
-
-void
+static void
 chatty_window_chat_list_select_first (ChattyWindow *self)
 {
   GtkListBoxRow *row;
@@ -528,7 +434,7 @@ chatty_window_chat_list_select_first (ChattyWindow *self)
     gtk_list_box_select_row (GTK_LIST_BOX(self->chats_listbox), row);
     window_chat_row_activated_cb (GTK_LIST_BOX(self->chats_listbox), row, self);
   } else {
-    chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
+    window_set_item (self, NULL);
   }
 }
 
@@ -542,12 +448,7 @@ window_delete_buddy_clicked_cb (ChattyWindow *self)
   int response;
 
   g_assert (CHATTY_IS_WINDOW (self));
-
-  if (!self->selected_item) {
-    chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
-
-    g_return_if_reached ();
-  }
+  g_return_if_fail (self->selected_item);
 
   name = chatty_item_get_name (CHATTY_ITEM (self->selected_item));
 
@@ -583,13 +484,23 @@ window_delete_buddy_clicked_cb (ChattyWindow *self)
   response = gtk_dialog_run (GTK_DIALOG (dialog));
 
   if (response == GTK_RESPONSE_OK) {
-    chatty_history_delete_chat (CHATTY_CHAT (self->selected_item));
-    chatty_pp_chat_delete (CHATTY_PP_CHAT (self->selected_item));
+    chatty_history_delete_chat (chatty_manager_get_history (self->manager),
+                                CHATTY_CHAT (self->selected_item));
+    if (CHATTY_IS_PP_CHAT (self->selected_item)) {
+      chatty_pp_chat_delete (CHATTY_PP_CHAT (self->selected_item));
+    } else if (CHATTY_IS_MM_CHAT (self->selected_item)) {
+      chatty_mm_chat_delete (CHATTY_MM_CHAT (self->selected_item));
+    } else {
+      g_return_if_reached ();
+    }
 
     window_set_item (self, NULL);
-    chatty_window_chat_list_select_first (self);
+    gtk_widget_hide (self->call_button);
+    gtk_widget_set_sensitive (self->header_sub_menu_button, FALSE);
+    chatty_chat_view_set_chat (CHATTY_CHAT_VIEW (self->chat_view), NULL);
 
-    chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
+    if (!hdy_leaflet_get_folded (HDY_LEAFLET (self->content_box)))
+      chatty_window_chat_list_select_first (self);
   }
 
   gtk_widget_destroy (dialog);
@@ -599,122 +510,102 @@ window_delete_buddy_clicked_cb (ChattyWindow *self)
 static void
 window_leave_chat_clicked_cb (ChattyWindow *self)
 {
-  PurpleBlistNode *node;
-
   g_assert (CHATTY_IS_WINDOW (self));
 
-  if (!self->selected_item) {
-    chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
+  if (self->selected_item) {
+    ChattyAccount *account;
 
-    g_return_if_reached ();
-  }
-
-  node = (PurpleBlistNode *)chatty_pp_chat_get_purple_buddy (CHATTY_PP_CHAT (self->selected_item));
-
-  if (!node)
-    node = (PurpleBlistNode *)chatty_pp_chat_get_purple_chat (CHATTY_PP_CHAT (self->selected_item));
-
-  if (node) {
-    purple_blist_node_set_bool (node, "chatty-autojoin", FALSE);
-    purple_conversation_destroy (chatty_pp_chat_get_purple_conv (CHATTY_PP_CHAT (self->selected_item)));
+    account = chatty_chat_get_account (CHATTY_CHAT (self->selected_item));
+    chatty_account_leave_chat_async (account,
+                                     CHATTY_CHAT (self->selected_item),
+                                     NULL, NULL);
   }
 
   window_set_item (self, NULL);
   chatty_window_chat_list_select_first (self);
-  chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
 }
 
+static void
+write_contact_cb (GObject      *object,
+                  GAsyncResult *result,
+                  gpointer      user_data)
+{
+  g_autoptr(ChattyWindow) self = user_data;
+  ChattyPpChat *chat = CHATTY_PP_CHAT (object);
+  g_autoptr(GError) error = NULL;
+  GtkWidget *dialog;
+
+  g_assert (CHATTY_IS_WINDOW (self));
+  g_assert (CHATTY_IS_PP_CHAT (chat));
+
+  if (chatty_pp_chat_save_to_contacts_finish (chat, result, &error))
+    return;
+
+  dialog = gtk_message_dialog_new (GTK_WINDOW (self),
+                                   GTK_DIALOG_MODAL,
+                                   GTK_MESSAGE_WARNING,
+                                   GTK_BUTTONS_CLOSE,
+                                   _("Error saving contact: %s"), error->message);
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+}
+
+static void
+write_eds_contact_cb (GObject      *object,
+                      GAsyncResult *result,
+                      gpointer      user_data)
+{
+  g_autoptr(ChattyWindow) self = user_data;
+  g_autoptr(GError) error = NULL;
+  GtkWidget *dialog;
+
+  g_assert (CHATTY_IS_WINDOW (self));
+
+  if (chatty_eds_write_contact_finish (result, &error))
+    return;
+
+  dialog = gtk_message_dialog_new (GTK_WINDOW (self),
+                                   GTK_DIALOG_MODAL,
+                                   GTK_MESSAGE_WARNING,
+                                   GTK_BUTTONS_CLOSE,
+                                   _("Error saving contact: %s"), error->message);
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+}
 
 static void
 window_add_contact_clicked_cb (ChattyWindow *self)
 {
-  PurpleAccount      *account;
-  PurpleConversation *conv;
-  PurpleBuddy        *buddy;
-  const char         *who;
-  g_autofree gchar   *number = NULL;
-
   g_assert (CHATTY_IS_WINDOW (self));
   g_return_if_fail (self->selected_item);
 
-  buddy = chatty_pp_chat_get_purple_buddy (CHATTY_PP_CHAT (self->selected_item));
-  g_return_if_fail (buddy != NULL);
+  if (CHATTY_IS_PP_CHAT (self->selected_item)) {
+    chatty_pp_chat_save_to_contacts_async (CHATTY_PP_CHAT (self->selected_item),
+                                           write_contact_cb, g_object_ref (self));
+  } else if (CHATTY_IS_MM_CHAT (self->selected_item)) {
+    const char *phone;
 
-  conv = chatty_pp_chat_get_purple_conv (CHATTY_PP_CHAT (self->selected_item));
-
-  account = purple_conversation_get_account (conv);
-  purple_account_add_buddy (account, buddy);
-  purple_blist_node_remove_setting (PURPLE_BLIST_NODE(buddy), "chatty-unknown-contact");
-  purple_blist_node_set_bool (PURPLE_BLIST_NODE (buddy), "chatty-notifications", TRUE);
-
-  if (chatty_item_get_protocols (CHATTY_ITEM (self->selected_item)) == CHATTY_PROTOCOL_SMS) {
-    ChattyEds *chatty_eds;
-    ChattyContact *contact = NULL;
-
-    chatty_eds = chatty_manager_get_eds (self->manager);
-
-    who = purple_buddy_get_name (buddy);
-
-    number = chatty_utils_check_phonenumber (who, chatty_settings_get_country_iso_code (self->settings));
-
-    if (number)
-      contact = chatty_eds_find_by_number (chatty_eds, number);
-
-    if (!contact)
-      chatty_dbus_gc_write_contact (who, number);
+    phone = chatty_chat_get_chat_name (CHATTY_CHAT (self->selected_item));
+    chatty_eds_write_contact_async ("", phone,
+                                    write_eds_contact_cb,
+                                    g_object_ref (self));
   }
 
   gtk_widget_hide (self->menu_add_contact_button);
 }
 
-
-static void
-window_add_in_contacts_clicked_cb (ChattyWindow *self)
-{
-  PurpleBuddy      *buddy;
-  PurpleContact    *contact;
-  const char       *who;
-  const char       *alias;
-  g_autofree gchar *number = NULL;
-
-  g_assert (CHATTY_IS_WINDOW (self));
-  g_return_if_fail (self->selected_item);
-
-  buddy = chatty_pp_chat_get_purple_buddy (CHATTY_PP_CHAT (self->selected_item));
-  g_return_if_fail (buddy != NULL);
-
-  who = purple_buddy_get_name (buddy);
-  contact = purple_buddy_get_contact (buddy);
-  alias = purple_contact_get_alias (contact);
-
-  number = chatty_utils_check_phonenumber (who, chatty_settings_get_country_iso_code (self->settings));
-
-  chatty_dbus_gc_write_contact (alias, number);
-}
-
-
 static void
 window_show_chat_info_clicked_cb (ChattyWindow *self)
 {
-  ChattyChat *chat;
-  GtkWidget *dialog;
+  ChattyInfoDialog *dialog;
 
   g_assert (CHATTY_IS_WINDOW (self));
   g_return_if_fail (CHATTY_IS_CHAT (self->selected_item));
 
-  chat = CHATTY_CHAT (self->selected_item);
-
-  if (chatty_chat_is_im (chat)) {
-    dialog = chatty_user_info_dialog_new (GTK_WINDOW (self));
-    chatty_user_info_dialog_set_chat (CHATTY_USER_INFO_DIALOG (dialog), chat);
-  } else {
-    dialog = chatty_muc_info_dialog_new (GTK_WINDOW (self));
-    chatty_muc_info_dialog_set_chat (CHATTY_MUC_INFO_DIALOG (dialog), chat);
-  }
+  dialog = CHATTY_INFO_DIALOG (self->chat_info_dialog);
 
+  chatty_info_dialog_set_chat (dialog, CHATTY_CHAT (self->selected_item));
   gtk_dialog_run (GTK_DIALOG (dialog));
-
-  gtk_widget_destroy (dialog);
 }
 
 static void
@@ -730,21 +621,6 @@ chatty_window_show_settings_dialog (ChattyWindow *self)
   gtk_widget_destroy (dialog);
 }
 
-
-static void
-chatty_window_show_new_muc_dialog (ChattyWindow *self)
-{
-  GtkWidget *dialog;
-
-  g_assert (CHATTY_IS_WINDOW (self));
-
-  dialog = chatty_new_muc_dialog_new (GTK_WINDOW (self));
-  gtk_dialog_run (GTK_DIALOG (dialog));
-
-  gtk_widget_destroy (dialog);
-}
-
-
 /* Copied from chatty-dialogs.c written by Andrea Schäfer <mosibasu@me.com> */
 static void
 chatty_window_show_about_dialog (ChattyWindow *self)
@@ -783,7 +659,7 @@ chatty_window_show_about_dialog (ChattyWindow *self)
                          "version", GIT_VERSION,
                          "comments", _("An SMS and XMPP messaging client"),
                          "website", "https://source.puri.sm/Librem5/chatty",
-                         "copyright", "© 2018–2020 Purism SPC",
+                         "copyright", "© 2018–2021 Purism SPC",
                          "license-type", GTK_LICENSE_GPL_3_0,
                          "authors", authors,
                          "artists", artists,
@@ -792,32 +668,6 @@ chatty_window_show_about_dialog (ChattyWindow *self)
                          NULL);
 }
 
-
-static void
-chatty_window_show_new_chat_dialog (ChattyWindow *self)
-{
-  /* XXX: Not used */
-  gtk_dialog_run (GTK_DIALOG (self->new_chat_dialog));
-}
-
-void
-chatty_window_change_view (ChattyWindow      *self,
-                           ChattyWindowState  view)
-{
-  g_assert (CHATTY_IS_WINDOW (self));
-
-  switch (view) {
-    case CHATTY_VIEW_MESSAGE_LIST:
-      hdy_leaflet_set_visible_child_name (HDY_LEAFLET (self->content_box), "content");
-      break;
-    case CHATTY_VIEW_CHAT_LIST:
-      hdy_leaflet_set_visible_child_name (HDY_LEAFLET (self->content_box), "sidebar");
-      break;
-    default:
-      ;
-  }
-}
-
 static void
 window_active_protocols_changed_cb (ChattyWindow *self)
 {
@@ -827,13 +677,12 @@ window_active_protocols_changed_cb (ChattyWindow *self)
   g_assert (CHATTY_IS_WINDOW (self));
 
   protocols = chatty_manager_get_active_protocols (self->manager);
-  has_sms = !!(protocols & CHATTY_PROTOCOL_SMS);
-  has_im  = !!(protocols & ~CHATTY_PROTOCOL_SMS);
+  has_sms = !!(protocols & CHATTY_PROTOCOL_MMS_SMS);
+  has_im  = !!(protocols & ~CHATTY_PROTOCOL_MMS_SMS);
 
   gtk_widget_set_sensitive (self->header_add_chat_button, has_sms || has_im);
   gtk_widget_set_sensitive (self->menu_new_group_message_button, has_im);
-  gtk_widget_set_sensitive (self->menu_new_bulk_sms_button, has_sms);
-  
+
   gtk_filter_changed (self->chat_filter, GTK_FILTER_CHANGE_DIFFERENT);
   window_chat_changed_cb (self);
 }
@@ -842,18 +691,14 @@ static void
 window_chat_deleted_cb (ChattyWindow *self,
                         ChattyChat   *chat)
 {
-  GtkWidget *view;
-
   g_assert (CHATTY_IS_WINDOW (self));
   g_assert (CHATTY_IS_CHAT (chat));
 
-  if (self->selected_item == (gpointer)chat)
-    window_set_item (self, NULL);
+  if (self->selected_item != (gpointer)chat)
+    return;
 
-  view = window_get_view_for_chat (self, chat);
-
-  if (view)
-    gtk_widget_destroy (view);
+  window_set_item (self, NULL);
+  chatty_chat_view_set_chat (CHATTY_CHAT_VIEW (self->chat_view), NULL);
 }
 
 static void
@@ -892,17 +737,7 @@ chatty_window_constructed (GObject *object)
     gtk_window_maximize (window);
 
   self->new_chat_dialog = chatty_new_chat_dialog_new (GTK_WINDOW (self));
-
-  hdy_leaflet_set_visible_child_name (HDY_LEAFLET (self->content_box), "sidebar");
-
-  hdy_search_bar_connect_entry (HDY_SEARCH_BAR(self->chats_search_bar),
-                                GTK_ENTRY (self->chats_search_entry));
-
-  gtk_widget_set_sensitive (GTK_WIDGET (self->header_sub_menu_button), FALSE);
-
-  chatty_window_change_view (self, CHATTY_VIEW_CHAT_LIST);
-
-  gtk_widget_set_sensitive (self->header_add_chat_button, FALSE);
+  self->chat_info_dialog = chatty_info_dialog_new (GTK_WINDOW (self));
 
   self->chat_filter = gtk_custom_filter_new ((GtkCustomFilterFunc)window_chat_name_matches,
                                              g_object_ref (self),
@@ -911,7 +746,7 @@ chatty_window_constructed (GObject *object)
                                                   self->chat_filter);
   gtk_list_box_bind_model (GTK_LIST_BOX (self->chats_listbox),
                            G_LIST_MODEL (self->filter_model),
-                           (GtkListBoxCreateWidgetFunc)window_chat_list_row_new,
+                           (GtkListBoxCreateWidgetFunc)chatty_list_row_new,
                            g_object_ref(self), g_object_unref);
 
   g_signal_connect_object (gtk_filter_list_model_get_model (self->filter_model),
@@ -967,13 +802,13 @@ chatty_window_class_init (ChattyWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, sub_header_label);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, sub_header_icon);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, menu_add_contact_button);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, menu_add_in_contacts_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, menu_new_message_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, menu_new_group_message_button);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, menu_new_bulk_sms_button);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, header_chat_info_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, header_add_chat_button);
+  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, call_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, header_sub_menu_button);
+  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, leave_button);
+  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, delete_button);
 
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, search_button);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, chats_search_bar);
@@ -988,11 +823,7 @@ chatty_window_class_init (ChattyWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, chat_list_view);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, chats_listbox);
 
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, convs_notebook);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, overlay_icon);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, overlay_label_1);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, overlay_label_2);
-  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, overlay_label_3);
+  gtk_widget_class_bind_template_child (widget_class, ChattyWindow, chat_view);
   gtk_widget_class_bind_template_child (widget_class, ChattyWindow, header_chat_list_new_msg_popover);
 
   gtk_widget_class_bind_template_callback (widget_class, notify_fold_cb);
@@ -1002,13 +833,11 @@ chatty_window_class_init (ChattyWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, window_back_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_show_chat_info_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_add_contact_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, window_add_in_contacts_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_leave_chat_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_delete_buddy_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, window_call_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_search_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_chat_row_activated_cb);
-  gtk_widget_class_bind_template_callback (widget_class, chatty_window_show_new_muc_dialog);
-  gtk_widget_class_bind_template_callback (widget_class, chatty_window_show_new_chat_dialog);
   gtk_widget_class_bind_template_callback (widget_class, chatty_window_show_settings_dialog);
   gtk_widget_class_bind_template_callback (widget_class, chatty_window_show_about_dialog);
 }
@@ -1026,15 +855,6 @@ chatty_window_init (ChattyWindow *self)
   g_signal_connect_object (self->manager, "chat-deleted",
                            G_CALLBACK (window_chat_deleted_cb), self,
                            G_CONNECT_SWAPPED);
-
-  g_signal_connect_after (G_OBJECT (self->convs_notebook),
-                          "switch-page",
-                          G_CALLBACK (window_notebook_after_switch_cb), self);
-
-  if (purple_prefs_get_bool (CHATTY_PREFS_ROOT "/conversations/show_tabs"))
-    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (self->convs_notebook), TRUE);
-  else
-    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (self->convs_notebook), FALSE);
 }
 
 
@@ -1053,31 +873,10 @@ void
 chatty_window_set_uri (ChattyWindow *self,
                        const char   *uri)
 {
-  g_autoptr(ChattyChat) chat = NULL;
-  ChattyChat    *item;
-  ChattyEds     *chatty_eds;
-  ChattyContact *contact;
-  PurpleAccount *account;
-  ChattyPpBuddy *pp_buddy;
-  PurpleBuddy   *buddy;
-  char          *who = NULL;
-  const char    *alias;
-
-  account = purple_accounts_find ("SMS", "prpl-mm-sms");
-
-  if (!purple_account_is_connected (account))
-    return;
+  g_autofree char *who = NULL;
 
   who = chatty_utils_check_phonenumber (uri, chatty_settings_get_country_iso_code (self->settings));
 
-  chatty_eds = chatty_manager_get_eds (self->manager);
-  contact = chatty_eds_find_by_number (chatty_eds, who);
-
-  if (contact)
-    alias = chatty_item_get_name (CHATTY_ITEM (contact));
-  else
-    alias = who;
-
   if (!who) {
     GtkWidget *dialog;
 
@@ -1091,91 +890,74 @@ chatty_window_set_uri (ChattyWindow *self,
 
     return;
   }
-  g_return_if_fail (who != NULL);
 
-  buddy = purple_find_buddy (account, who);
-
-  if (!buddy) {
-    buddy = purple_buddy_new (account, who, alias);
-
-    purple_blist_add_buddy (buddy, NULL, NULL, NULL);
-  }
-
-  pp_buddy = chatty_pp_buddy_get_object (buddy);
-
-  if (pp_buddy && contact)
-    chatty_pp_buddy_set_contact (pp_buddy, contact);
-
-  chat = (ChattyChat *)chatty_pp_chat_new_im_chat (account, buddy);
-  item = chatty_manager_add_chat (chatty_manager_get_default (), chat);
-
-  purple_blist_node_set_bool (PURPLE_BLIST_NODE(buddy), "chatty-autojoin", TRUE);
-
-  chatty_conv_im_with_buddy (account, g_strdup (who));
+  if (!chatty_manager_set_uri (self->manager, uri))
+    return;
 
   gtk_widget_hide (self->new_chat_dialog);
-
-  chatty_window_change_view (self, CHATTY_VIEW_MESSAGE_LIST);
-  g_signal_emit_by_name (item, "changed");
-
-  g_free (who);
 }
 
 ChattyChat *
 chatty_window_get_active_chat (ChattyWindow *self)
 {
-  GtkWidget *child;
-  gint current_page;
+  GtkWidget *widget;
 
   g_return_val_if_fail (CHATTY_IS_WINDOW (self), NULL);
 
-  current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (self->convs_notebook));
+  widget = gtk_window_get_focus (GTK_WINDOW (self));
 
-  if (current_page == -1)
-    return NULL;
+  if (widget && gtk_widget_has_focus (widget))
+    return chatty_chat_view_get_chat (CHATTY_CHAT_VIEW (self->chat_view));
 
-  child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (self->convs_notebook), current_page);
-  if (!gtk_widget_is_drawable (child))
-    return NULL;
-
-  return chatty_chat_view_get_chat (CHATTY_CHAT_VIEW (child));
+  return NULL;
 }
 
 void
 chatty_window_open_chat (ChattyWindow *self,
                          ChattyChat   *chat)
 {
-  GtkWidget *view;
-  int page_num;
+  gboolean can_delete;
 
   g_return_if_fail (CHATTY_IS_WINDOW (self));
   g_return_if_fail (CHATTY_IS_CHAT (chat));
+  g_debug ("opening item of type: %s, protocol: %d",
+           G_OBJECT_TYPE_NAME (chat),
+           chatty_item_get_protocols (CHATTY_ITEM (chat)));
 
-  view = window_get_view_for_chat (self, chat);
+  chatty_chat_view_set_chat (CHATTY_CHAT_VIEW (self->chat_view), chat);
+  window_set_item (self, CHATTY_ITEM (chat));
+  window_chat_changed_cb (self);
 
-  if (!view) {
+  gtk_widget_set_visible (self->leave_button, !CHATTY_IS_MM_CHAT (chat));
+  can_delete = CHATTY_IS_PP_CHAT (chat) || CHATTY_IS_MM_CHAT (chat);
+  gtk_widget_set_visible (self->delete_button, can_delete);
+  hdy_leaflet_set_visible_child (HDY_LEAFLET (self->content_box), self->chat_view);
+  gtk_widget_hide (self->menu_add_contact_button);
+  gtk_widget_hide (self->call_button);
+
+  if (chatty_window_get_active_chat (self))
+    chatty_chat_set_unread_count (chat, 0);
+
+  if (CHATTY_IS_MM_CHAT (chat)) {
+    GListModel *users;
     const char *name;
-    g_auto(GStrv) split = NULL;
-    g_autofree char *label = NULL;
 
-    view  = chatty_chat_view_new ();
-    name  = chatty_item_get_name (CHATTY_ITEM (chat));
-    split = g_strsplit (name, "@", -1);
-    label = g_strdup_printf ("%s %s", split[0], " >");
+    users = chatty_chat_get_users (chat);
+    name = chatty_chat_get_chat_name (chat);
 
-    chatty_chat_view_set_chat (CHATTY_CHAT_VIEW (view), chat);
-    gtk_widget_show (view);
+    if (g_list_model_get_n_items (users) == 1 &&
+        chatty_utils_username_is_valid (name, CHATTY_PROTOCOL_MMS_SMS)) {
+      g_autoptr(ChattyMmBuddy) buddy = NULL;
+      g_autoptr(GAppInfo) app_info = NULL;
 
-    gtk_container_add (GTK_CONTAINER (self->convs_notebook), view);
-    gtk_notebook_set_tab_label_text (GTK_NOTEBOOK(self->convs_notebook), view, label);
+      app_info = g_app_info_get_default_for_uri_scheme ("tel");
+      buddy = g_list_model_get_item (users, 0);
 
-    chatty_manager_load_more_chat (chatty_manager_get_default (), chat, LAZY_LOAD_INITIAL_MSGS_LIMIT);
+      if (app_info)
+        gtk_widget_show (self->call_button);
+
+      if (!chatty_mm_buddy_get_contact (buddy))
+        gtk_widget_show (self->menu_add_contact_button);
+    }
   }
-
-  page_num = gtk_notebook_page_num (GTK_NOTEBOOK (self->convs_notebook), view);
-  gtk_notebook_set_current_page (GTK_NOTEBOOK (self->convs_notebook), page_num);
-  hdy_leaflet_set_visible_child (HDY_LEAFLET (self->content_box), self->convs_notebook);
-
-  chatty_chat_set_unread_count (chat, 0);
-  gtk_window_present (GTK_WINDOW (self));
 }
