@@ -19,11 +19,12 @@
 #include "chatty-pp-chat.h"
 #include "users/chatty-contact.h"
 #include "contrib/gtk.h"
+#include "users/chatty-mm-account.h"
 #include "users/chatty-pp-account.h"
+#include "matrix/chatty-ma-account.h"
 #include "chatty-list-row.h"
-#include "chatty-dbus.h"
 #include "chatty-utils.h"
-#include "chatty-icons.h"
+#include "chatty-phone-utils.h"
 #include "chatty-new-chat-dialog.h"
 
 
@@ -62,7 +63,7 @@ struct _ChattyNewChatDialog
   GtkFilter *filter;
   char      *search_str;
 
-  ChattyPpAccount *selected_account;
+  ChattyAccount   *selected_account;
   ChattyManager   *manager;
   ChattyProtocol   active_protocols;
 
@@ -77,32 +78,28 @@ struct _ChattyNewChatDialog
 G_DEFINE_TYPE (ChattyNewChatDialog, chatty_new_chat_dialog, GTK_TYPE_DIALOG)
 
 
-static gboolean
-new_chat_str_is_phone (const char *str)
-{
-  guint length;
-
-  if (!str || !*str)
-    return FALSE;
-
-  if (*str == '+')
-    str++;
-
-  length = strlen (str);
-
-  if (length >= 3 && strspn (str, "0123456789") == length)
-    return TRUE;
-
-  return FALSE;
-}
-
 static void
 dialog_active_protocols_changed_cb (ChattyNewChatDialog *self)
 {
+  ChattyAccount *mm_account;
+  ChattyProtocol protocol;
+  gboolean valid;
+
   g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
 
   self->active_protocols = chatty_manager_get_active_protocols (self->manager);
   gtk_filter_changed (self->filter, GTK_FILTER_CHANGE_DIFFERENT);
+
+  protocol = CHATTY_PROTOCOL_MMS_SMS;
+  valid = protocol == chatty_utils_username_is_valid (self->search_str, protocol);
+  mm_account = chatty_manager_get_mm_account (self->manager);
+  valid = valid && chatty_account_get_status (mm_account) == CHATTY_CONNECTED;
+  gtk_widget_set_visible (self->new_contact_row, valid);
+
+  if (valid || g_list_model_get_n_items (G_LIST_MODEL (self->slice_model)) > 0)
+    gtk_stack_set_visible_child (GTK_STACK (self->contact_list_stack), self->contact_list_view);
+  else
+    gtk_stack_set_visible_child (GTK_STACK (self->contact_list_stack), self->empty_search_view);
 }
 
 
@@ -117,11 +114,11 @@ dialog_filter_item_cb (ChattyItem          *item,
       return FALSE;
 
   if (CHATTY_IS_PP_BUDDY (item)) {
-    PurpleAccount *account;
+    ChattyAccount *account;
 
     account = chatty_pp_buddy_get_account (CHATTY_PP_BUDDY (item));
 
-    if (!purple_account_is_connected (account))
+    if (chatty_account_get_status (account) != CHATTY_CONNECTED)
       return FALSE;
   }
 
@@ -135,13 +132,27 @@ dialog_filter_item_cb (ChattyItem          *item,
 
     account = chatty_chat_get_account (CHATTY_CHAT (item));
 
-    if (chatty_account_get_status (account) != CHATTY_CONNECTED)
+    if (!account || chatty_account_get_status (account) != CHATTY_CONNECTED)
       return FALSE;
   }
 
   return chatty_item_matches (item, self->search_str, self->active_protocols, TRUE);
 }
 
+static void
+new_chat_list_changed_cb (ChattyNewChatDialog *self)
+{
+  guint n_items;
+
+  g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
+
+  n_items = g_list_model_get_n_items (G_LIST_MODEL (self->slice_model));
+
+  if (n_items > 0 || gtk_widget_get_visible (self->new_contact_row))
+    gtk_stack_set_visible_child (GTK_STACK (self->contact_list_stack), self->contact_list_view);
+  else
+    gtk_stack_set_visible_child (GTK_STACK (self->contact_list_stack), self->empty_search_view);
+}
 
 static void
 chatty_new_chat_dialog_update_new_contact_row (ChattyNewChatDialog *self)
@@ -150,6 +161,7 @@ chatty_new_chat_dialog_update_new_contact_row (ChattyNewChatDialog *self)
 
   self->dummy_contact = g_object_new (CHATTY_TYPE_CONTACT, NULL);
   chatty_contact_set_name (self->dummy_contact, _("Send To"));
+  g_object_set_data (G_OBJECT (self->dummy_contact), "dummy", GINT_TO_POINTER (TRUE));
 }
 
 static void
@@ -185,6 +197,8 @@ open_contacts_finish_cb (GObject      *object,
   g_assert (CHATTY_IS_EDS (chatty_eds));
 
   chatty_eds_open_contacts_app_finish (chatty_eds, result, &error);
+  gtk_widget_set_sensitive (self->add_in_contacts_button, TRUE);
+  gtk_stack_set_visible_child_name (GTK_STACK (self->new_chat_stack), "view-new-chat");
 
   if (!error)
     return;
@@ -207,11 +221,10 @@ add_in_contacts_button_clicked_cb (ChattyNewChatDialog *self)
   g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
 
   chatty_eds = chatty_manager_get_eds (self->manager);
+  gtk_widget_set_sensitive (self->add_in_contacts_button, FALSE);
   chatty_eds_open_contacts_app (chatty_eds,
                                 self->cancellable,
                                 open_contacts_finish_cb, self);
-
-  gtk_stack_set_visible_child_name (GTK_STACK (self->new_chat_stack), "view-new-chat");
 }
 
 
@@ -240,10 +253,11 @@ static void
 contact_search_entry_changed_cb (ChattyNewChatDialog *self,
                                  GtkEntry            *entry)
 {
-  PurpleAccount *account;
+  ChattyAccount *account;
   g_autofree char *old_needle = NULL;
   const char *str;
   GtkFilterChange change;
+  ChattyProtocol protocol;
   gboolean valid;
 
   g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
@@ -274,9 +288,10 @@ contact_search_entry_changed_cb (ChattyNewChatDialog *self,
   chatty_list_row_set_item (CHATTY_LIST_ROW (self->new_contact_row),
                             CHATTY_ITEM (self->dummy_contact));
 
-  valid = new_chat_str_is_phone (self->search_str);
-  account = purple_accounts_find ("SMS", "prpl-mm-sms");
-  valid = valid && purple_account_is_connected (account);
+  protocol = CHATTY_PROTOCOL_MMS_SMS;
+  valid = protocol == chatty_utils_username_is_valid (self->search_str, protocol);
+  account = chatty_manager_get_mm_account (self->manager);
+  valid = valid && chatty_account_get_status (account) == CHATTY_CONNECTED;
   gtk_widget_set_visible (self->new_contact_row, valid);
 
   if (valid || g_list_model_get_n_items (G_LIST_MODEL (self->slice_model)) > 0)
@@ -300,28 +315,25 @@ contact_row_activated_cb (ChattyNewChatDialog *self,
 static void
 add_contact_button_clicked_cb (ChattyNewChatDialog *self)
 {
-  PurpleAccount     *account;
-  char              *who;
-  const char        *alias;
-  g_autoptr(GError)  err = NULL;
+  GPtrArray *buddies;
+  const char *who, *alias;
 
   g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
 
   if (!gtk_widget_get_sensitive (self->add_contact_button))
     return;
 
-  account = chatty_pp_account_get_account (self->selected_account);
+  buddies = g_ptr_array_new_full (1, g_free);
 
-  who = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->contact_name_entry)));
+  who = gtk_entry_get_text (GTK_ENTRY (self->contact_name_entry));
   alias = gtk_entry_get_text (GTK_ENTRY (self->contact_alias_entry));
+  chatty_pp_account_add_buddy (CHATTY_PP_ACCOUNT (self->selected_account), who, alias);
 
-  chatty_pp_account_add_buddy (self->selected_account, who, alias);
-  chatty_conv_im_with_buddy (account, g_strdup (who));
+  g_ptr_array_add (buddies, g_strdup (who));
+  chatty_account_start_direct_chat_async (self->selected_account, buddies, NULL, NULL);
 
   gtk_widget_hide (GTK_WIDGET (self));
 
-  g_free (who);
-
   gtk_entry_set_text (GTK_ENTRY (self->contact_name_entry), "");
   gtk_entry_set_text (GTK_ENTRY (self->contact_alias_entry), "");
 
@@ -345,7 +357,7 @@ account_list_row_activated_cb (ChattyNewChatDialog *self,
                                GtkListBoxRow       *row,
                                GtkListBox          *box)
 {
-  ChattyPpAccount *account;
+  ChattyAccount *account;
   GtkWidget       *prefix_radio;
 
   g_assert (CHATTY_IS_NEW_CHAT_DIALOG (self));
@@ -356,12 +368,10 @@ account_list_row_activated_cb (ChattyNewChatDialog *self,
   self->selected_account = account;
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefix_radio), TRUE);
-
-  if (chatty_item_is_sms (CHATTY_ITEM (account))) {
+  if (CHATTY_IS_MM_ACCOUNT (account))
     chatty_new_chat_set_edit_mode (self, FALSE);
-  } else {
+  else
     chatty_new_chat_set_edit_mode (self, TRUE);
-  }
 }
 
 
@@ -370,25 +380,22 @@ chatty_new_chat_name_check (ChattyNewChatDialog *self,
                             GtkEntry            *entry,
                             GtkWidget           *button)
 {
-  PurpleAccount *account;
-  PurpleBuddy   *buddy = NULL;
-  const char    *name;
+  const char *name;
+  ChattyProtocol protocol, valid_protocol;
+  gboolean valid = TRUE;
 
   g_return_if_fail (CHATTY_IS_NEW_CHAT_DIALOG (self));
 
   name = gtk_entry_get_text (entry);
 
-  account = chatty_pp_account_get_account (self->selected_account);
+  protocol = chatty_item_get_protocols (CHATTY_ITEM (self->selected_account));
+  valid_protocol = chatty_utils_username_is_valid (name, protocol);
+  valid = protocol == valid_protocol;
 
-  if ((*name != '\0') && account) {
-    buddy = purple_find_buddy (account, name);
-  }
+  if (valid)
+    valid &= !chatty_account_buddy_exists (CHATTY_ACCOUNT (self->selected_account), name);
 
-  if ((*name != '\0') && !buddy) {
-    gtk_widget_set_sensitive (button, TRUE);
-  } else {
-    gtk_widget_set_sensitive (button, FALSE);
-  }
+  gtk_widget_set_sensitive (button, valid);
 }
 
 
@@ -412,7 +419,7 @@ chatty_new_chat_set_edit_mode (ChattyNewChatDialog *self,
 
 static void
 chatty_new_chat_add_account_to_list (ChattyNewChatDialog *self,
-                                     ChattyPpAccount     *account)
+                                     ChattyAccount     *account)
 {
   HdyActionRow   *row;
   GtkWidget      *prefix_radio_button;
@@ -420,15 +427,10 @@ chatty_new_chat_add_account_to_list (ChattyNewChatDialog *self,
 
   g_return_if_fail (CHATTY_IS_NEW_CHAT_DIALOG (self));
 
-  row = HDY_ACTION_ROW (hdy_action_row_new ());
-  g_object_set_data (G_OBJECT (row),
-                     "row-account",
-                     (gpointer)account);
-
   protocol = chatty_item_get_protocols (CHATTY_ITEM (account));
 
   // TODO list supported protocols here
-  if (protocol & ~(CHATTY_PROTOCOL_SMS |
+  if (protocol & ~(CHATTY_PROTOCOL_MMS_SMS |
                    CHATTY_PROTOCOL_XMPP |
                    CHATTY_PROTOCOL_MATRIX |
                    CHATTY_PROTOCOL_TELEGRAM |
@@ -436,9 +438,17 @@ chatty_new_chat_add_account_to_list (ChattyNewChatDialog *self,
                    CHATTY_PROTOCOL_THREEPL))
     return;
 
-  if (chatty_account_get_status (CHATTY_ACCOUNT (account)) == CHATTY_DISCONNECTED) {
+  if (chatty_account_get_status (account) <= CHATTY_DISCONNECTED &&
+      !CHATTY_IS_MM_ACCOUNT (account))
     return;
-  }
+
+  /* We don't handle native matrix accounts here  */
+  if (CHATTY_IS_MA_ACCOUNT (account))
+    return;
+
+  row = HDY_ACTION_ROW (hdy_action_row_new ());
+  gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), TRUE);
+  g_object_set_data (G_OBJECT (row), "row-account", (gpointer)account);
 
   prefix_radio_button = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (self->dummy_prefix_radio));
   gtk_widget_show (GTK_WIDGET (prefix_radio_button));
@@ -450,7 +460,8 @@ chatty_new_chat_add_account_to_list (ChattyNewChatDialog *self,
                      (gpointer)prefix_radio_button);
 
   hdy_action_row_add_prefix (row, GTK_WIDGET (prefix_radio_button ));
-  hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (row), chatty_account_get_username (CHATTY_ACCOUNT (account)));
+  hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (row),
+                                 chatty_item_get_username (CHATTY_ITEM (account)));
 
   gtk_container_add (GTK_CONTAINER (self->accounts_list), GTK_WIDGET (row));
 
@@ -479,6 +490,7 @@ chatty_new_chat_account_list_clear (GtkWidget *list)
 static void
 chatty_new_chat_populate_account_list (ChattyNewChatDialog *self)
 {
+  ChattyAccount *mm_account;
   GListModel   *model;
   HdyActionRow *row;
   guint         n_items;
@@ -492,13 +504,17 @@ chatty_new_chat_populate_account_list (ChattyNewChatDialog *self)
   n_items = g_list_model_get_n_items (model);
 
   for (guint i = 0; i < n_items; i++) {
-    g_autoptr(ChattyPpAccount) account = NULL;
+    g_autoptr(ChattyAccount) account = NULL;
 
     account = g_list_model_get_item (model, i);
 
     chatty_new_chat_add_account_to_list (self, account);
   }
 
+  /* Add sms account */
+  mm_account = chatty_manager_get_mm_account (self->manager);
+  chatty_new_chat_add_account_to_list (self, mm_account);
+
   row = HDY_ACTION_ROW(gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->accounts_list), 0));
 
   if (row) {
@@ -608,20 +624,26 @@ chatty_new_chat_dialog_init (ChattyNewChatDialog *self)
   self->dummy_prefix_radio = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (NULL));
 
   self->manager = g_object_ref (chatty_manager_get_default ());
-  self->filter = gtk_custom_filter_new ((GtkCustomFilterFunc)dialog_filter_item_cb, self, NULL);
-  g_signal_connect_object (self->manager, "notify::active-protocols",
-                           G_CALLBACK (dialog_active_protocols_changed_cb), self, G_CONNECT_SWAPPED);
-  dialog_active_protocols_changed_cb (self);
 
   sorter = gtk_custom_sorter_new ((GCompareDataFunc)chatty_item_compare, NULL, NULL);
   sort_model = gtk_sort_list_model_new (chatty_manager_get_contact_list (self->manager), sorter);
+
+  self->filter = gtk_custom_filter_new ((GtkCustomFilterFunc)dialog_filter_item_cb, self, NULL);
   filter_model = gtk_filter_list_model_new (G_LIST_MODEL (sort_model), self->filter);
+
   self->slice_model = gtk_slice_list_model_new (G_LIST_MODEL (filter_model), 0, ITEMS_COUNT);
+  g_signal_connect_object (self->slice_model, "items-changed",
+                           G_CALLBACK (new_chat_list_changed_cb), self,
+                           G_CONNECT_SWAPPED);
   gtk_list_box_bind_model (GTK_LIST_BOX (self->chats_listbox),
                            G_LIST_MODEL (self->slice_model),
                            (GtkListBoxCreateWidgetFunc)chatty_list_contact_row_new,
                            NULL, NULL);
 
+  g_signal_connect_object (self->manager, "notify::active-protocols",
+                           G_CALLBACK (dialog_active_protocols_changed_cb), self, G_CONNECT_SWAPPED);
+  dialog_active_protocols_changed_cb (self);
+
   chatty_new_chat_dialog_update_new_contact_row (self);
 }
 
