@@ -19,8 +19,8 @@
 #include <gtk/gtk.h>
 #include <libebook-contacts/libebook-contacts.h>
 
-#include "chatty-icons.h"
 #include "chatty-settings.h"
+#include "chatty-utils.h"
 #include "chatty-contact.h"
 #include "chatty-contact-private.h"
 
@@ -75,15 +75,18 @@ chatty_contact_matches (ChattyItem     *item,
 
   g_assert (CHATTY_IS_CONTACT (self));
 
-  value = chatty_contact_get_value (self);
+  value = chatty_item_get_username (item);
   protocol = chatty_item_get_protocols (item);
 
-  if (protocol == CHATTY_PROTOCOL_SMS &&
-      protocols & CHATTY_PROTOCOL_SMS) {
+  if (protocol == CHATTY_PROTOCOL_MMS_SMS &&
+      protocols & CHATTY_PROTOCOL_MMS_SMS) {
     ChattySettings *settings;
     const char *country;
     EPhoneNumberMatch match;
 
+    if (strstr (value, needle))
+      return TRUE;
+
     settings = chatty_settings_get_default ();
     country = chatty_settings_get_country_iso_code (settings);
     match = e_phone_number_compare_strings_with_region (value, needle, country, NULL);
@@ -122,12 +125,26 @@ chatty_contact_get_name (ChattyItem *item)
   return "";
 }
 
+static const char *
+chatty_contact_get_username (ChattyItem *item)
+{
+  ChattyContact *self = (ChattyContact *)item;
+
+  g_assert (CHATTY_IS_CONTACT (self));
+
+  if (!self->value && self->attribute)
+    self->value = e_vcard_attribute_get_value (self->attribute);
+
+  if (self->value)
+    return self->value;
+
+  return "";
+}
 
 static GdkPixbuf *
 chatty_contact_get_avatar (ChattyItem *item)
 {
   ChattyContact *self = (ChattyContact *)item;
-  g_autoptr(GError) error = NULL;
   EContactPhoto *photo;
 
   g_assert (CHATTY_IS_CONTACT (self));
@@ -159,9 +176,11 @@ chatty_contact_get_avatar (ChattyItem *item)
     data = e_contact_photo_get_inlined (photo, &len);
 
     if (data)
-      self->avatar = chatty_icon_pixbuf_from_data (data, len);
+      self->avatar = chatty_utils_get_pixbuf_from_data (data, len);
   }
 
+  e_contact_photo_free (photo);
+
   return self->avatar;
 }
 
@@ -188,6 +207,7 @@ chatty_contact_dispose (GObject *object)
 {
   ChattyContact *self = (ChattyContact *)object;
 
+  g_clear_object (&self->e_contact);
   g_clear_object (&self->avatar);
   g_clear_pointer (&self->attribute, e_vcard_attribute_free);
   g_clear_pointer (&self->name, g_free);
@@ -208,6 +228,7 @@ chatty_contact_class_init (ChattyContactClass *klass)
   item_class->get_protocols = chatty_contact_get_protocols;
   item_class->matches  = chatty_contact_matches;
   item_class->get_name = chatty_contact_get_name;
+  item_class->get_username = chatty_contact_get_username;
   item_class->get_avatar = chatty_contact_get_avatar;
   item_class->get_avatar_async  = chatty_contact_get_avatar_async;
 }
@@ -258,31 +279,6 @@ chatty_contact_set_name (ChattyContact *self,
   self->name = g_strdup (name);
 }
 
-/**
- * chatty_contact_get_value:
- * @self: A #ChattyContact
- *
- * Get the value stored in @self. It can be a phone
- * number, an XMPP ID, etc.
- * Also see chatty_contact_get_value_type().
- *
- * Returns: (transfer none): The value of @self.
- * Or an empty string if no value.
- */
-const char *
-chatty_contact_get_value (ChattyContact *self)
-{
-  g_return_val_if_fail (CHATTY_IS_CONTACT (self), NULL);
-
-  if (!self->value && self->attribute)
-    self->value = e_vcard_attribute_get_value (self->attribute);
-
-  if (self->value)
-    return self->value;
-
-  return "";
-}
-
 void
 chatty_contact_set_value (ChattyContact *self,
                           const char    *value)
@@ -342,6 +338,39 @@ chatty_contact_get_uid (ChattyContact *self)
   return "";
 }
 
+gboolean
+chatty_contact_is_exact_match (ChattyContact  *self,
+                               const char     *value,
+                               ChattyProtocol  protocols)
+{
+  const char *contact_value;
+  ChattyProtocol protocol;
+
+  g_assert (CHATTY_IS_CONTACT (self));
+
+  contact_value = chatty_item_get_username (CHATTY_ITEM (self));
+  protocol = chatty_item_get_protocols (CHATTY_ITEM (self));
+
+  if (protocol & (CHATTY_PROTOCOL_MMS_SMS | CHATTY_PROTOCOL_MMS) &&
+      protocols & (CHATTY_PROTOCOL_MMS_SMS | CHATTY_PROTOCOL_MMS)) {
+    ChattySettings *settings;
+    const char *country;
+    EPhoneNumberMatch match;
+
+    if (g_str_equal (contact_value, value))
+      return TRUE;
+
+    settings = chatty_settings_get_default ();
+    country = chatty_settings_get_country_iso_code (settings);
+    match = e_phone_number_compare_strings_with_region (contact_value, value, country, NULL);
+
+    if (match == E_PHONE_NUMBER_MATCH_EXACT ||
+        match == E_PHONE_NUMBER_MATCH_NATIONAL)
+      return TRUE;
+  }
+
+  return FALSE;
+}
 
 /**
  * chatty_contact_clear_cache:
@@ -363,5 +392,5 @@ chatty_contact_is_dummy (ChattyContact *self)
 {
   g_return_val_if_fail (CHATTY_IS_CONTACT (self), TRUE);
 
-  return self->e_contact == NULL;
+  return !!g_object_get_data (G_OBJECT (self), "dummy");
 }
