@@ -11,13 +11,19 @@
 
 #define G_LOG_DOMAIN "chatty-pp-account"
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <handy.h>
 #include <purple.h>
 
-#include "chatty-config.h"
 #include "chatty-settings.h"
 #include "chatty-account.h"
 #include "chatty-window.h"
+#include "chatty-pp-chat.h"
 #include "chatty-pp-account.h"
+#include "chatty-log.h"
 
 /**
  * SECTION: chatty-pp-account
@@ -38,12 +44,15 @@ struct _ChattyPpAccount
   gchar          *username;
   gchar          *server_url;
   GListStore     *buddy_list;
+  HdyValueObject *device_fp;
+  GListStore     *fp_list;
 
   PurpleAccount  *pp_account;
   PurpleStoredImage *pp_avatar;
   GdkPixbuf         *avatar;
   guint           connect_id;
   ChattyProtocol  protocol;
+  gboolean        has_encryption;
 
   ChattyPpAccountFeatures features;
 };
@@ -122,7 +131,7 @@ account_connect (ChattyPpAccount *self)
   if (!purple_status_is_online (pp_status))
     return G_SOURCE_REMOVE;
 
-  g_debug ("connecting to %s", chatty_account_get_username (CHATTY_ACCOUNT (self)));
+  CHATTY_DEBUG (chatty_item_get_username (CHATTY_ITEM (self)), "connecting to");
   purple_account_connect (self->pp_account);
 
   return G_SOURCE_REMOVE;
@@ -131,7 +140,7 @@ account_connect (ChattyPpAccount *self)
 static void
 chatty_pp_account_create (ChattyPpAccount *self)
 {
-  const char *protocol_id;
+  const char *protocol_id = NULL;
   ChattyProtocol protocol;
 
   g_assert (CHATTY_IS_PP_ACCOUNT (self));
@@ -142,8 +151,6 @@ chatty_pp_account_create (ChattyPpAccount *self)
     protocol_id = "prpl-jabber";
   else if (protocol == CHATTY_PROTOCOL_MATRIX)
     protocol_id = "prpl-matrix";
-  else if (protocol == CHATTY_PROTOCOL_SMS)
-    protocol_id = "prpl-mm-sms";
   else if (protocol == CHATTY_PROTOCOL_TELEGRAM)
     protocol_id = "prpl-telegram";
 
@@ -153,11 +160,6 @@ chatty_pp_account_create (ChattyPpAccount *self)
       {
         purple_account_set_string (self->pp_account, "home_server", self->server_url);
       }
-    else if (protocol == CHATTY_PROTOCOL_SMS)
-      {
-        purple_account_set_password (self->pp_account, NULL);
-        purple_account_set_remember_password (self->pp_account, TRUE);
-      }
 }
 
 static void
@@ -176,8 +178,6 @@ chatty_pp_load_protocol (ChattyPpAccount *self)
     protocol = CHATTY_PROTOCOL_XMPP;
   else if (g_str_equal (protocol_id, "prpl-matrix"))
     protocol = CHATTY_PROTOCOL_MATRIX;
-  else if (g_str_equal (protocol_id, "prpl-mm-sms"))
-    protocol = CHATTY_PROTOCOL_SMS;
   else if (g_str_equal (protocol_id, "prpl-telegram"))
     protocol = CHATTY_PROTOCOL_TELEGRAM;
   else if (g_str_equal (protocol_id, "prpl-delta"))
@@ -213,25 +213,31 @@ chatty_pp_account_get_status (ChattyAccount *account)
   return CHATTY_DISCONNECTED;
 }
 
-static const gchar *
-chatty_pp_account_get_username (ChattyAccount *account)
+static GListModel *
+chatty_pp_account_get_buddies (ChattyAccount *account)
 {
   ChattyPpAccount *self = (ChattyPpAccount *)account;
 
   g_assert (CHATTY_IS_PP_ACCOUNT (self));
 
-  return purple_account_get_username (self->pp_account);
+  return G_LIST_MODEL (self->buddy_list);
 }
 
-static void
-chatty_pp_account_set_username (ChattyAccount *account,
-                                const char    *username)
+static gboolean
+chatty_pp_account_buddy_exists (ChattyAccount *account,
+                                const char    *buddy_username)
 {
   ChattyPpAccount *self = (ChattyPpAccount *)account;
+  PurpleBuddy *buddy;
 
   g_assert (CHATTY_IS_PP_ACCOUNT (self));
 
-  purple_account_set_username (self->pp_account, username);
+  if (!buddy_username || !*buddy_username)
+    return FALSE;
+
+  buddy = purple_find_buddy (self->pp_account, buddy_username);
+
+  return buddy != NULL;
 }
 
 static gboolean
@@ -284,6 +290,43 @@ chatty_pp_account_set_password (ChattyAccount *account,
     purple_account_set_password (self->pp_account, password);
 }
 
+static void
+chatty_pp_account_connect (ChattyAccount *account,
+                           gboolean       delay)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  g_clear_handle_id (&self->connect_id, g_source_remove);
+
+  if (!delay)
+    account_connect (self);
+  else
+    self->connect_id = g_timeout_add (RECONNECT_DELAY,
+                                      G_SOURCE_FUNC (account_connect),
+                                      self);
+}
+
+static void
+chatty_pp_account_disconnect (ChattyAccount *account)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+  g_autofree char *password = NULL;
+  ChattyStatus status;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  status = chatty_account_get_status (CHATTY_ACCOUNT (self));
+
+  if (status <= CHATTY_DISCONNECTED)
+    return;
+
+  password = g_strdup (chatty_account_get_password (CHATTY_ACCOUNT (self)));
+  purple_account_disconnect (self->pp_account);
+  chatty_account_set_password (CHATTY_ACCOUNT (self), password);
+}
+
 static void
 chatty_pp_account_set_remember_password (ChattyAccount *account,
                                          gboolean       remember)
@@ -316,6 +359,172 @@ chatty_pp_account_delete (ChattyAccount *account)
   purple_accounts_delete (self->pp_account);
 }
 
+static HdyValueObject *
+chatty_pp_account_get_device_fp (ChattyAccount *account)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  return self->device_fp;
+}
+
+static GListModel *
+chatty_pp_account_get_fp_list (ChattyAccount *account)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  return G_LIST_MODEL (self->fp_list);
+}
+
+static void
+get_fp_list_cb (int         error,
+                GHashTable *fp_table,
+                gpointer    user_data)
+{
+  g_autoptr(GList) key_list = NULL;
+  g_autoptr(GTask) task = user_data;
+  ChattyPpAccount *self;
+
+  g_assert (G_IS_TASK (task));
+
+  self = g_task_get_source_object (task);
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  if (error || !fp_table) {
+    g_warning ("has error: %d", !!error);
+    g_task_return_boolean (task, FALSE);
+
+    return;
+  }
+
+  key_list = g_hash_table_get_keys (fp_table);
+  g_clear_object (&self->device_fp);
+  g_list_store_remove_all (self->fp_list);
+
+  for (GList *item = key_list; item; item = item->next) {
+    const char *fp = NULL;
+
+    fp = g_hash_table_lookup (fp_table, item->data);
+
+    g_debug ("DeviceId: %i fingerprint: %s", *((guint32 *) item->data),
+             fp ? fp : "(no session)");
+
+    if (fp) {
+      char *id;
+
+      id = g_strdup_printf ("%u", *((guint32 *) item->data));
+      /* The first fingerprint is current device fingerprint */
+      if (!self->device_fp) {
+        self->device_fp = hdy_value_object_new_string (fp);
+        g_object_set_data_full (G_OBJECT (self->device_fp), "device-id", id, g_free);
+      } else {
+        g_autoptr(HdyValueObject) object = NULL;
+
+        object = hdy_value_object_new_string (fp);
+        g_object_set_data_full (G_OBJECT (object), "device-id", id, g_free);
+        g_list_store_append (self->fp_list, object);
+      }
+    }
+  }
+
+  g_task_return_boolean (task, TRUE);
+}
+
+static void
+chatty_pp_account_load_fp_async (ChattyAccount       *account,
+                                 GAsyncReadyCallback  callback,
+                                 gpointer             user_data)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+  GTask *task;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  if (!self->has_encryption) {
+    CHATTY_ACCOUNT_CLASS (chatty_pp_account_parent_class)->load_fp_async (account, callback, user_data);
+
+    return;
+  }
+
+  task = g_task_new (self, NULL, callback, user_data);
+
+  purple_signal_emit (purple_plugins_get_handle (),
+                      "lurch-fp-list",
+                      self->pp_account,
+                      get_fp_list_cb,
+                      task);
+}
+
+static void
+chatty_pp_account_join_chat_async (ChattyAccount       *account,
+                                   ChattyChat          *chat,
+                                   GAsyncReadyCallback  callback,
+                                   gpointer             user_data)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+  g_autoptr(GTask) task = NULL;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+  g_return_if_fail (CHATTY_IS_PP_CHAT (chat));
+
+  chatty_pp_chat_join (CHATTY_PP_CHAT (chat));
+
+  /* Assume we succeeded! */
+  task = g_task_new (self, NULL, callback, user_data);
+  g_task_return_boolean (task, TRUE);
+}
+
+static void
+chatty_pp_account_leave_chat_async (ChattyAccount       *account,
+                                    ChattyChat          *chat,
+                                    GAsyncReadyCallback  callback,
+                                    gpointer             user_data)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+  g_autoptr(GTask) task = NULL;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  chatty_pp_chat_leave (CHATTY_PP_CHAT (chat));
+
+  /* Assume we succeeded! */
+  task = g_task_new (self, NULL, callback, user_data);
+  g_task_return_boolean (task, TRUE);
+}
+
+static void
+chatty_pp_account_start_direct_chat_async (ChattyAccount       *account,
+                                           GPtrArray           *buddies,
+                                           GAsyncReadyCallback  callback,
+                                           gpointer             user_data)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)account;
+  PurpleConversation *conv;
+  const char *name;
+  g_autoptr(GTask) task = NULL;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  g_return_if_fail (buddies->len == 1);
+  g_return_if_fail (purple_account_is_connected (self->pp_account));
+
+  task = g_task_new (self, NULL, callback, user_data);
+  name = buddies->pdata[0];
+  conv = purple_find_conversation_with_account (PURPLE_CONV_TYPE_IM, name, self->pp_account);
+
+  if (!conv)
+    conv = purple_conversation_new (PURPLE_CONV_TYPE_IM, self->pp_account, name);
+
+  purple_conversation_present (conv);
+  g_ptr_array_unref (buddies);
+
+  /* Just assume we succeeded, what else can we do! */
+  g_task_return_boolean (task, TRUE);
+}
+
 static gboolean
 chatty_pp_account_get_remember_password (ChattyAccount *account)
 {
@@ -366,6 +575,27 @@ chatty_pp_account_set_name (ChattyItem *item,
   purple_account_set_alias (self->pp_account, name);
 }
 
+static const char *
+chatty_pp_account_get_username (ChattyItem *item)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)item;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  return purple_account_get_username (self->pp_account);
+}
+
+static void
+chatty_pp_account_set_username (ChattyItem *item,
+                                const char *username)
+{
+  ChattyPpAccount *self = (ChattyPpAccount *)item;
+
+  g_assert (CHATTY_IS_PP_ACCOUNT (self));
+
+  purple_account_set_username (self->pp_account, username);
+}
+
 static GdkPixbuf *
 chatty_icon_from_data (const guchar *buf,
                        gsize         size)
@@ -520,6 +750,9 @@ chatty_pp_account_finalize (GObject *object)
   if (self->pp_avatar)
     purple_imgstore_unref (self->pp_avatar);
 
+  g_list_store_remove_all (self->fp_list);
+  g_clear_object (&self->fp_list);
+  g_clear_object (&self->device_fp);
   g_clear_object (&self->buddy_list);
   g_clear_object (&self->avatar);
   g_free (self->username);
@@ -542,21 +775,31 @@ chatty_pp_account_class_init (ChattyPpAccountClass *klass)
   item_class->get_protocols = chatty_pp_account_get_protocols;
   item_class->get_name = chatty_pp_account_get_name;
   item_class->set_name = chatty_pp_account_set_name;
+  item_class->get_username = chatty_pp_account_get_username;
+  item_class->set_username = chatty_pp_account_set_username;
   item_class->get_avatar = chatty_pp_account_get_avatar;
   item_class->set_avatar_async = chatty_pp_account_set_avatar_async;
 
   account_class->get_protocol_name = chatty_pp_account_get_protocol_name;
   account_class->get_status   = chatty_pp_account_get_status;
-  account_class->get_username = chatty_pp_account_get_username;
-  account_class->set_username = chatty_pp_account_set_username;
+  account_class->get_buddies  = chatty_pp_account_get_buddies;
+  account_class->buddy_exists = chatty_pp_account_buddy_exists;
   account_class->get_enabled  = chatty_pp_account_get_enabled;
   account_class->set_enabled  = chatty_pp_account_set_enabled;
   account_class->get_password = chatty_pp_account_get_password;
   account_class->set_password = chatty_pp_account_set_password;
+  account_class->connect      = chatty_pp_account_connect;
+  account_class->disconnect   = chatty_pp_account_disconnect;
   account_class->get_remember_password = chatty_pp_account_get_remember_password;
   account_class->set_remember_password = chatty_pp_account_set_remember_password;
   account_class->save = chatty_pp_account_save;
   account_class->delete = chatty_pp_account_delete;
+  account_class->get_device_fp = chatty_pp_account_get_device_fp;
+  account_class->get_fp_list = chatty_pp_account_get_fp_list;
+  account_class->load_fp_async = chatty_pp_account_load_fp_async;
+  account_class->join_chat_async = chatty_pp_account_join_chat_async;
+  account_class->leave_chat_async = chatty_pp_account_leave_chat_async;
+  account_class->start_direct_chat_async = chatty_pp_account_start_direct_chat_async;
 
   properties[PROP_USERNAME] =
     g_param_spec_string ("username",
@@ -585,6 +828,7 @@ static void
 chatty_pp_account_init (ChattyPpAccount *self)
 {
   self->buddy_list = g_list_store_new (CHATTY_TYPE_PP_BUDDY);
+  self->fp_list = g_list_store_new (HDY_TYPE_VALUE_OBJECT);
 }
 
 /**
@@ -613,30 +857,41 @@ chatty_pp_account_get_object (PurpleAccount *account)
 ChattyPpAccount *
 chatty_pp_account_new (ChattyProtocol  protocol,
                        const char     *username,
-                       const char     *server_url)
+                       const char     *server_url,
+                       gboolean        has_encryption)
 {
-  g_return_val_if_fail (protocol & (CHATTY_PROTOCOL_SMS |
-                                    CHATTY_PROTOCOL_XMPP |
+  ChattyPpAccount *self;
+
+  g_return_val_if_fail (protocol & (CHATTY_PROTOCOL_XMPP |
                                     CHATTY_PROTOCOL_MATRIX |
                                     CHATTY_PROTOCOL_TELEGRAM), NULL);
   g_return_val_if_fail (username && *username, NULL);
 
-  return g_object_new (CHATTY_TYPE_PP_ACCOUNT,
+  self = g_object_new (CHATTY_TYPE_PP_ACCOUNT,
                        "protocols", protocol,
                        "username", username,
                        "server-url", server_url,
                        NULL);
+  self->has_encryption = !!has_encryption;
+
+  return self;
 }
 
 ChattyPpAccount *
-chatty_pp_account_new_purple (PurpleAccount *account)
+chatty_pp_account_new_purple (PurpleAccount *account,
+                              gboolean       has_encryption)
 {
+  ChattyPpAccount *self;
+
   g_return_val_if_fail (account, NULL);
   g_return_val_if_fail (!account->ui_data, NULL);
 
-  return g_object_new (CHATTY_TYPE_PP_ACCOUNT,
+  self = g_object_new (CHATTY_TYPE_PP_ACCOUNT,
                        "purple-account", account,
                        NULL);
+  self->has_encryption = !!has_encryption;
+
+  return self;
 }
 
 ChattyPpBuddy *
@@ -674,12 +929,45 @@ chatty_pp_account_add_purple_buddy (ChattyPpAccount *self,
   return buddy;
 }
 
-GListModel *
-chatty_pp_account_get_buddy_list (ChattyPpAccount *self)
+ChattyChat *
+chatty_pp_account_join_chat (ChattyPpAccount *self,
+                             const char      *chat_id,
+                             const char      *room_alias,
+                             const char      *user_alias,
+                             const char      *password)
 {
-  g_return_val_if_fail (CHATTY_IS_PP_ACCOUNT (self), NULL);
+  PurplePluginProtocolInfo *info;
+  PurpleConnection *gc;
+  PurpleGroup *group;
+  PurpleChat *chat;
+  GHashTable *hash = NULL;
 
-  return G_LIST_MODEL (self->buddy_list);
+  g_return_val_if_fail (CHATTY_IS_PP_ACCOUNT (self), NULL);
+  g_return_val_if_fail (chat_id && *chat_id, NULL);
+
+  if (!purple_account_is_connected (self->pp_account))
+    return NULL;
+
+  gc = purple_account_get_connection (self->pp_account);
+  info = PURPLE_PLUGIN_PROTOCOL_INFO (purple_connection_get_prpl (gc));
+
+  if (info->chat_info_defaults != NULL)
+    hash = info->chat_info_defaults (gc, chat_id);
+
+  if (user_alias && *user_alias)
+    g_hash_table_replace (hash, "handle", g_strdup (user_alias));
+
+  chat = purple_chat_new (self->pp_account, chat_id, hash);
+  if ((group = purple_find_group ("Chats")) == NULL) {
+    group = purple_group_new ("Chats");
+    purple_blist_add_group (group, NULL);
+  }
+
+  purple_blist_add_chat (chat, group, NULL);
+  purple_blist_alias_chat (chat, room_alias);
+  purple_blist_node_set_bool ((PurpleBlistNode*)chat, "chatty-autojoin", TRUE);
+
+  return chatty_pp_chat_get_object (chat);
 }
 
 /* XXX: a helper API till the dust settles */
@@ -712,56 +1000,6 @@ chatty_pp_account_get_protocol_id (ChattyPpAccount *self)
   return id ? id : "";
 }
 
-/**
- * chatty_pp_account_connect:
- * @self: A #ChattyPpAccount
- * @delay: Whether to delay connection
- *
- * connection to @self.  If @delay is %TRUE, the connection
- * is initiated after some delay, which can be useful when
- * trying to connect after a connection failure.
- *
- * If the account is not enabled, or if account status is
- * set to offline, or if already connected, the function
- * simply returns.
- */
-void
-chatty_pp_account_connect (ChattyPpAccount *self,
-                           gboolean         delay)
-{
-  g_return_if_fail (CHATTY_IS_PP_ACCOUNT (self));
-
-  g_clear_handle_id (&self->connect_id, g_source_remove);
-
-  if (!delay)
-    account_connect (self);
-  else
-    self->connect_id = g_timeout_add (RECONNECT_DELAY,
-                                      G_SOURCE_FUNC (account_connect),
-                                      self);
-}
-
-void
-chatty_pp_account_disconnect (ChattyPpAccount *self)
-{
-  g_autofree char *password = NULL;
-  ChattyStatus status;
-
-  g_return_if_fail (CHATTY_IS_PP_ACCOUNT (self));
-
-  if (chatty_item_is_sms (CHATTY_ITEM (self)))
-    return;
-
-  status = chatty_account_get_status (CHATTY_ACCOUNT (self));
-
-  if (status == CHATTY_DISCONNECTED)
-    return;
-
-  password = g_strdup (chatty_account_get_password (CHATTY_ACCOUNT (self)));
-  purple_account_disconnect (self->pp_account);
-  chatty_account_set_password (CHATTY_ACCOUNT (self), password);
-}
-
 void
 chatty_pp_account_set_features (ChattyPpAccount *self,
                                 ChattyPpAccountFeatures features)
