@@ -11,12 +11,15 @@
 
 #define G_LOG_DOMAIN "chatty-pp-buddy"
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #define _GNU_SOURCE
 #include <string.h>
 #include <purple.h>
 #include <glib/gi18n.h>
 
-#include "chatty-config.h"
 #include "chatty-settings.h"
 #include "chatty-account.h"
 #include "chatty-pp-account.h"
@@ -126,9 +129,6 @@ chatty_add_new_buddy (ChattyPpBuddy *self)
 
   g_debug ("%s: %s ", __func__, purple_buddy_get_name (self->pp_buddy));
 
-  if (chatty_item_get_protocols (CHATTY_ITEM (self)) != CHATTY_PROTOCOL_SMS)
-    purple_account_add_buddy_with_invite (self->pp_account, self->pp_buddy, NULL);
-
   conv = purple_find_conversation_with_account (PURPLE_CONV_TYPE_IM,
                                                 self->username,
                                                 self->pp_account);
@@ -232,6 +232,30 @@ chatty_pp_buddy_set_name (ChattyItem *item,
   purple_blist_alias_contact (contact, self->name);
 }
 
+static const char *
+chatty_pp_buddy_get_username (ChattyItem *item)
+{
+  ChattyPpBuddy *self = (ChattyPpBuddy *)item;
+  const char *name;
+
+  g_assert (CHATTY_IS_PP_BUDDY (self));
+
+  if (self->contact)
+    return chatty_item_get_username (CHATTY_ITEM (self->contact));
+
+  if (self->chat_buddy && self->chat_buddy->name)
+    return self->chat_buddy->name;
+
+  if (!self->pp_buddy)
+    return "";
+
+  name = purple_buddy_get_name (self->pp_buddy);
+
+  if (!name)
+    name = "";
+
+  return name;
+}
 
 static gboolean
 load_icon (gpointer user_data)
@@ -367,7 +391,7 @@ static void
 chatty_pp_buddy_constructed (GObject *object)
 {
   ChattyPpBuddy *self = (ChattyPpBuddy *)object;
-  ChattyPpAccount *account;
+  ChattyAccount *account;
   GListModel *model;
   gboolean has_pp_buddy;
 
@@ -388,7 +412,7 @@ chatty_pp_buddy_constructed (GObject *object)
   g_object_add_weak_pointer (G_OBJECT (self), (gpointer *)&PURPLE_BLIST_NODE (self->pp_buddy)->ui_data);
 
   account = self->pp_buddy->account->ui_data;
-  model = chatty_pp_account_get_buddy_list (account);
+  model = chatty_account_get_buddies (account);
   g_list_store_append (G_LIST_STORE (model), self);
 
   if (!has_pp_buddy)
@@ -421,6 +445,7 @@ chatty_pp_buddy_class_init (ChattyPpBuddyClass *klass)
   item_class->matches  = chatty_pp_buddy_matches;
   item_class->get_name = chatty_pp_buddy_get_name;
   item_class->set_name = chatty_pp_buddy_set_name;
+  item_class->get_username = chatty_pp_buddy_get_username;
   item_class->get_avatar = chatty_pp_buddy_get_avatar;
 
   properties[PROP_PURPLE_ACCOUNT] =
@@ -480,23 +505,19 @@ chatty_pp_buddy_get_object (PurpleBuddy *buddy)
   return node->ui_data;
 }
 
-PurpleAccount *
+ChattyAccount *
 chatty_pp_buddy_get_account (ChattyPpBuddy *self)
 {
+  PurpleAccount *account;
+
   g_return_val_if_fail (CHATTY_IS_PP_BUDDY (self), NULL);
 
   if (self->chat_buddy && self->conv)
-    return self->conv->account;
+    account = self->conv->account;
+  else
+    account = purple_buddy_get_account (self->pp_buddy);
 
-  return purple_buddy_get_account (self->pp_buddy);
-}
-
-PurpleConversation *
-chatty_pp_buddy_get_chat (ChattyPpBuddy *self)
-{
-  g_return_val_if_fail (CHATTY_IS_PP_BUDDY (self), NULL);
-
-  return self->conv;
+  return CHATTY_ACCOUNT (chatty_pp_account_get_object (account));
 }
 
 void
@@ -517,42 +538,6 @@ chatty_pp_buddy_get_buddy (ChattyPpBuddy *self)
   return self->pp_buddy;
 }
 
-
-/**
- * chatty_pp_buddy_get_id:
- * @self: a #ChattyPpBuddy
- *
- * Get the user id of @self.  For XMPP, this
- * is the XMPP name (eg: name@example.com).
- * For SMS buddy, this is the phone number.
- *
- * Returns: (transfer none): the id of Buddy.
- * or an empty string if not found or on error.
- */
-const char *
-chatty_pp_buddy_get_id (ChattyPpBuddy *self)
-{
-  const char *name;
-
-  g_return_val_if_fail (CHATTY_IS_PP_BUDDY (self), "");
-
-  if (self->contact)
-    return chatty_contact_get_value (self->contact);
-
-  if (self->chat_buddy && self->chat_buddy->name)
-    return self->chat_buddy->name;
-
-  if (!self->pp_buddy)
-    return "";
-
-  name = purple_buddy_get_name (self->pp_buddy);
-
-  if (!name)
-    name = "";
-
-  return name;
-}
-
 ChattyContact *
 chatty_pp_buddy_get_contact (ChattyPpBuddy *self)
 {
