@@ -5,7 +5,7 @@
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
-#define G_LOG_DOMAIN "chatty-xeps"
+#define G_LOG_DOMAIN "chatty-xeps-0313"
 
 #include <glib.h>
 #include <prpl.h>
@@ -21,6 +21,7 @@
 #include "chatty-pp-chat.h"
 #include "chatty-manager.h"
 #include "chatty-settings.h"
+#include "chatty-log.h"
 
 #define NS_FWDv0 "urn:xmpp:forward:0"
 #define NS_SIDv0 "urn:xmpp:sid:0"
@@ -177,7 +178,7 @@ static void
 chatty_mam_ctx_del(PurpleAccount *pa)
 {
   g_return_if_fail(pa != NULL);
-  g_debug("Cleaning context for %s", purple_account_get_username(pa));
+  CHATTY_DEBUG (purple_account_get_username (pa), "Cleaning context for ");
   g_hash_table_remove(ht_mam_ctx, purple_account_get_username(pa));
 }
 
@@ -477,9 +478,11 @@ cb_chatty_mam_bare_info(PurpleConnection *pc,
     mamq->js = js;
     mamq->id = g_strdup(qid);
     if(g_strcmp0(bare, purple_account_get_username(pa))) {
+      ChattyManager *manager = chatty_manager_get_default ();
       time_t ts;
 
-      ts = chatty_history_get_last_message_time (purple_account_get_username(pa), bare);
+      ts = chatty_history_get_last_message_time (chatty_manager_get_history (manager),
+                                                 purple_account_get_username(pa), bare);
       // For MUC we're getting all messages so last history ts is ok
       if(ts>0)
         dt = g_date_time_new_from_unix_utc(ts);
@@ -501,8 +504,8 @@ cb_chatty_mam_bare_info(PurpleConnection *pc,
     }
     mamq->start = g_date_time_format(dt,"%FT%TZ");
     g_date_time_unref(dt);
-    g_debug ("Server supports MAM %s on %s; Querying by %s from %s after %s",
-                                    var, bare, qid, mamq->start, mamq->after);
+    CHATTY_DEBUG (bare, "Server supports MAM %s; Querying by %s from %s after %s, id:",
+                  var, qid, mamq->start, mamq->after);
     // Request MAM backlog
     chatty_mam_query_archive(mamq);
     // Also - request preferences and correct them if required
@@ -654,14 +657,19 @@ cb_chatty_mam_msg_received (PurpleConnection *pc,
   node_sid    = xmlnode_get_child_with_namespace (msg, "stanza-id", NS_SIDv0);
 
   if(mamc->cur_msg != NULL) {
+    g_autofree char *str = NULL;
+
+    str = g_strdup_printf ("%s %s", from, to);
+
     // Skip resubmission - break the loop
-    g_debug ("Received resubmission %s %s %s", id, from, to);
+    CHATTY_DEBUG (str, "Received resubmission %s", id);
     return FALSE;
   }
 
   user = purple_account_get_username (pa);
 
   if(node_result != NULL || node_sid != NULL) {
+    ChattyManager *manager = chatty_manager_get_default ();
     int dts;
     const char *msg_type;
     if(node_result != NULL) {
@@ -703,14 +711,17 @@ cb_chatty_mam_msg_received (PurpleConnection *pc,
       // store the SID in history at the least - to know where to start.
       message = msg;
       peer = from;
-      g_debug ("Received forward id %s from %s", stanza_id, peer);
+      CHATTY_DEBUG (peer, "Received forward id %s from", stanza_id);
     }
     // check history and drop the dup
     msg_type = xmlnode_get_attrib(message, "type");
+
     if(from && msg_type && g_strcmp0(msg_type, "groupchat") == 0) {
-      dts = chatty_history_get_chat_timestamp (stanza_id, from);
+      dts = chatty_history_get_chat_timestamp (chatty_manager_get_history (manager),
+                                               stanza_id, from);
     } else {
-      dts = chatty_history_get_im_timestamp (stanza_id, user);
+      dts = chatty_history_get_im_timestamp (chatty_manager_get_history (manager),
+                                             stanza_id, user);
     }
     if(dts < INT_MAX) {
       g_debug ("Message id %s for acc %s is already stored on %d", stanza_id, user, dts);
@@ -741,7 +752,7 @@ cb_chatty_mam_msg_received (PurpleConnection *pc,
       if(node_oid) {
         const char *uuid = xmlnode_get_attrib (node_oid, "id");
         if(uuid) {
-          dts = chatty_history_get_im_timestamp (uuid, user);
+          dts = chatty_history_get_im_timestamp (chatty_manager_get_history (manager), uuid, user);
           if(dts < INT_MAX) {
             g_debug ("Message id %s for acc %s is already stored on %d", uuid, user, dts);
             return TRUE; // note - true means stop processing
@@ -754,7 +765,7 @@ cb_chatty_mam_msg_received (PurpleConnection *pc,
     message = msg;
     peer = from;
   }
-  g_debug ("Stealing parser for MAM, from %s at ID %s", peer, stanza_id);
+  CHATTY_DEBUG (peer, "Stealing parser for MAM, ID %s user:", stanza_id);
   /**
    * Before we resume message processing we need to pre-cook the message.
    * If there's no body the whole server_got_stuff is skipped, so we may never
@@ -770,6 +781,7 @@ cb_chatty_mam_msg_received (PurpleConnection *pc,
     mamc->cur_msg->p.when = purple_str_to_time (stamp, TRUE, NULL, NULL, NULL);
   jabber_message_parse (js, message);
   if(stanza_id != NULL || mamc->cur_msg->p.what != NULL) {
+    ChattyManager *manager = chatty_manager_get_default ();
     PurpleConvMessage *pcm = &(mamc->cur_msg->p);
     PurpleConversation *conv = mamc->cur_msg->conv;
     g_autoptr(ChattyMessage) chat_message = NULL;
@@ -793,11 +805,24 @@ cb_chatty_mam_msg_received (PurpleConnection *pc,
     if (!stanza_id)
       stanza_id = g_uuid_string_random ();
 
-    chat_message = chatty_message_new (NULL, NULL, pcm->what, stanza_id, pcm->when,
-                                       chatty_utils_direction_from_flag (pcm->flags), 0);
-    if (chat_message && pcm->who && !(flags & PURPLE_MESSAGE_SEND))
-      chatty_message_set_user_name (chat_message, who ? who : pcm->who);
-    chatty_history_add_message (conv->ui_data, chat_message);
+    {
+      g_autoptr(ChattyContact) contact = NULL;
+
+      contact = g_object_new (CHATTY_TYPE_CONTACT, NULL);
+      chatty_contact_set_name (contact, who);
+      chatty_contact_set_value (contact, who);
+      chat_message = chatty_message_new (CHATTY_ITEM (contact), pcm->what, stanza_id,
+                                         pcm->when, CHATTY_MESSAGE_HTML_ESCAPED,
+                                         chatty_utils_direction_from_flag (pcm->flags), 0);
+    }
+
+    if (conv)
+      chatty_history_add_message (chatty_manager_get_history (manager),
+                                  conv->ui_data, chat_message);
+    else
+      g_warning ("NULL conversation for : who: %s, message: %s",
+                  who ? who: pcm->who, pcm->what);
+
   }
   // Update last timestamp for account's archive
   if(mamq != NULL && mamq->to == NULL)
